###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# alignAce.py
from cistematic.programs import Program
import os, time, string
from cistematic.core.motif import Motif

class AlignACE(Program):
    alignacePath = Program.programRoot + '/alignace/'
    motifs = []
    argDict = {}
    
    def getSettings(self):
        return self.argDict
    
    def setSettings(self, settings):
        self.motifs = []
        self.argDict = settings
    
    def setGenExpOptions(self, optionArray):
        for option in optionArray:
            (optionTag, optionValue) = option.split(':')
            if optionTag == 'gcback':
                self.setgcBack(optionValue)
    
    def buildCommand(self):
        cmd = self.alignacePath + 'AlignACE -i ' + self.inputFilePath 
        
        for arg in self.argDict.keys():
            cmd = cmd + ' -' + arg + ' %s' % self.argDict[arg]
        print "cmd is %s" % (cmd)
        
        return cmd

    #set expect
    def setExpect(self, exp):
        self.argDict['expect'] = exp
    
    # set genome defaults
    def setGenome(self, gen):
        if gen == 'arabidopsis':
            self.setgcBack('0.35')
    
    # set gcback
    def setgcBack(self, gcb):
        self.argDict['gcback'] = gcb
        
    # set minpass
    def setMinpass(self, minp):
        self.argDict['minpass'] = minp
    
    # set seed
    def setSeed(self, sd):
        self.argDict['seed'] = sd
        
    # set numcols
    def setNumcols(self, ncols):
        self.argDict['numcols'] = ncols
    
    # set undersample
    def setUndersample(self, under):
        self.argDict['undersample'] = under
    
    # set oversample
    def setNumcols(self, over):
        self.argDict['oversample'] = over
    
    # run the program - preferrably after some of the other options have been set
    def run(self):
        startTime = time.time()
        self.contents = os.popen(AlignACE.buildCommand(self)).readlines()
        stopTime = time.time()
            
        print "\nThis run took %.3f - %.3f = %.3f seconds" % (startTime, stopTime, stopTime - startTime)

    def getMotifs(self):
    
        self.motifs = []
        index = 0
        nmotifs = 0
        matrixRow = {'A': 0, 'C': 1, 'G': 2, 'T': 3}
        
        for line in self.contents:
            if string.find(line,'MAP Score:') > -1:
                nmotifs += 1
 
        index = self.locateMotif(index)       
        for motif in range(0, nmotifs):
            thePWM = []
            consensus = ""
            
            words = self.contents[index].split()
            for ind in range(len(words[0])):
                thePWM.append([0.0, 0.0, 0.0, 0.0])
            
            seqNum = 0.0
            while string.find(self.contents[index],'MAP') < 0:
                if self.contents[index][0] in ('A', 'C', 'G', 'T'):
                    seqNum += 1.0
                    pos = 0
                    words = string.split(self.contents[index])
                    if 'N' in words[0]:
                        continue
                    for NT in words[0]:
                        thePWM[pos][matrixRow[NT]] += 1
                        pos += 1
                index += 1
            
            mapValue = self.contents[index]
            
            if seqNum > 0:
                for pos in range(len(thePWM)):
                    thePWM[pos][matrixRow['A']] /= seqNum
                    thePWM[pos][matrixRow['C']] /= seqNum
                    thePWM[pos][matrixRow['G']] /= seqNum
                    thePWM[pos][matrixRow['T']] /= seqNum

            self.motifs.append(Motif(self.tagID + '-ACE-' + str(motif + 1),'',thePWM, [], 1.0, mapValue))
            index += 6
        
        return self.motifs
    
    def locateMotif(self, startFrom):
        index = startFrom
        numLines = len(self.contents)
        
        while index < numLines and string.find(self.contents[index],'Motif ') < 0:
            index += 1
        
        if index < numLines:
            return index + 2
            
        return -1

