###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# cobind.py
from cistematic.programs import Program
import os, time, string
from cistematic.core.motif import Motif
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

#cobindPath = '/proj/genome/programs/cobind/'

class CoBind(Program):
    cobindPath =  Program.programRoot + '/cobind/'
    argDict = {}
    motifs = []
    
    def __init__(self, tag='', inp='', outp=''):
        self.tagID = ''
        Parent.__init__(self, tag, inp, outp)
        
        self.argDict['n'] = self.cobindPath + 'sample_files/background.fa'
        self.argDict['a'] = self.cobindPath + 'sample_files/alphabet'
        self.argDict['c'] = '1'
        self.argDict['V'] = '0'
        self.argDict['Z'] = '100'

    def getSettings(self):
        return self.argDict
    
    def setSettings(self, settings):
        self.motifs = []
        self.argDict = settings

    def buildCommand(self):
        cmd = self.cobindPath + 'bin/co-bind -p ' + self.inputFilePath 
        
        for arg in self.argDict.keys():
            cmd = cmd + ' -' + arg + ' ' + self.argDict[arg]
        #print "cmd is %s" % (cmd)
        
        return cmd

        
    #set background
    def setBackground(self, back):
        self.argDict['n'] = back
    
    # set max distance between sites
    def setMaxDistance(self, maxd):
        self.argDict['Z'] = maxd
        
    # set alphabet
    def setAlphabet(self, alpha):
        self.argDict['a'] = alpha
    
    # set genome defaults
    def setGenome(self, gen):
        if gen == 'arabidopsis':
            self.setBackground(cisRoot + '/A_thaliana/ara600.fsa')
    
    # run the program - preferrably after some of the other options have been set
    def run(self):
        startTime = time.time()
        self.contents = os.popen(CoBind.buildCommand(self)).readlines()
        stopTime = time.time()
            
        print "\nThis run took %.3f - %.3f = %.3f seconds" % (startTime, stopTime, stopTime - startTime)

    def getMotifs(self):
        index = 0
        nmotifs = 0
        matrixRow = {'A': 0, 'C': 1, 'G': 2, 'T': 3}
        
        self.motifs = []
        for motif in (1, 3):
            thePWM = []
            consensus = ""
            
            index = self.locateMotif(index)       
            words = self.contents[index].split('\t')
            motWord = words[1].split()
            for ind in range(len(motWord[0])):
                thePWM.append([0.0, 0.0, 0.0, 0.0])
            
            seqNum = 0.0
            while string.find(self.contents[index],'ave') < 0:
                seqNum += 1.0
                pos = 0
                words = self.contents[index].split('\t')
                motWord = words[1].split()

                print '%d\t%f: %s' % (motif, seqNum, motWord[0])
                for NT in motWord[0]:
                    thePWM[pos][matrixRow[NT]] += 1
                    pos += 1
                index += 1
            
            aveLog = self.contents[index]
            
            if seqNum > 0:
                for pos in range(len(thePWM)):
                    thePWM[pos][matrixRow['A']] /= seqNum
                    thePWM[pos][matrixRow['C']] /= seqNum
                    thePWM[pos][matrixRow['G']] /= seqNum
                    thePWM[pos][matrixRow['T']] /= seqNum

            self.motifs.append(Motif(self.tagID + '-COBIND-' + str(motif),'',thePWM, [], 1.0, aveLog))
        
        return self.motifs
    
    def locateMotif(self, startFrom):
        index = startFrom
        numLines = len(self.contents)
        
        while index < numLines and string.find(self.contents[index],'BEST_SITES') < 0:
            index += 1
        
        print self.contents[index]
        if index < numLines:
            return index + 2
            
        return -1

