###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# footprinter.py
from cistematic.programs import Program
import os, time, tempfile
from cistematic.core.motif import Motif

class FootPrinter(Program):
    footprinterPath = Program.programRoot + '/FootPrinter2.0/'
    tempfile.tempdir = Program.programRoot + '/workdir'
    treeFile = Program.programRoot + '/FootPrinter2.0/tree_of_life'
    motifLength  = 8
    maxMutations = 2
    maxMutationsPerBranch = 1
    motifs = []
    
    def getSettings(self):
        return (self.motifLength, self.maxMutations, self.maxMutationsPerBranch, self.treeFile)
    
    def setSettings(self, settings):
        self.motifs = []
        (self.motifLength, self.maxMutations, self.maxMutationsPerBranch, self.treeFile) = settings
    
    def setMotifLength(self, mLength):
        self.motifLength = mLength
    
    def setMaxMutations(self, mNum):
        self.maxMutations = mNum
    
    def setMaxMutationsPerBranch(self, mNum):
        self.maxMutationsPerBranch = mNum
    
    def setTreeFile(self, tFile):
        self.treeFile = tFile
    
    def buildCommand(self, tempDir):
        cmd = 'FOOTPRINTER_DIR=' + self.footprinterPath + ' ; export FOOTPRINTER_DIR; cd ' + tempDir + '; ' + self.footprinterPath + 'FootPrinter ' + self.inputFilePath +  ' ' + self.treeFile + ' -max_mutations ' + str(self.maxMutations) + ' -max_mutations_per_branch ' + str(self.maxMutationsPerBranch) + ' -size ' + str(self.motifLength) + ' -no_html -no_html_interactive -no_ps'
        
        #print "cmd is %s" % (cmd)
        
        return cmd

    # run the program - preferrably after some of the other options have been set
    def run(self):
        oldtempdir = tempfile.tempdir
        tempfile.tempdir = Parent.parentRoot + '/workdir'
        fpTempDir = tempfile.mktemp()
        tempfile.tempdir = oldtempdir
        
        os.mkdir(fpTempDir)
        startTime = time.time()
        self.contents = os.popen(FootPrinter.buildCommand(self, fpTempDir)).readlines()
        stopTime = time.time()

        resFile = open(self.inputFilePath + '.seq.txt', 'r')
        self.contents = resFile.readlines()
        resFile.close()
    
    # only getting motifs in top sequence, and only getting the consensus
    def getMotifs(self):
        index = 1
        pos = 0
        currentMotifID = 0
        currentMotif = ''
        motifPos = 0
        seqLine = self.contents[index]
        fileLength = len(self.contents)
        motDict = {}
        self.motifs = []
        
        while seqLine[0] != '\n':
            motifLine = self.contents[index + 2]
            for pos in range(len(seqLine)):
                motifChar = ord(motifLine[pos])
                if motifChar == 10:
                    continue
                elif motifChar == 32:
                    motifPos = 0
                    currentMotifID = 0
                    currentMotif = ''
                elif motifChar <> currentMotifID:
                    #print 'changing from %s to %s' % (chr(currentMotifID),chr(motifChar))
                    currentMotifID = motifChar
                    motifPos = 1
                    currentMotif = seqLine[pos]
                elif motifChar == currentMotifID: # we have a growing motif
                    motifPos += 1
                    currentMotif += seqLine[pos]
                    #print 'mot %s %d: %s' % (chr(motifChar), motifPos, currentMotif)
                    if motifPos == self.motifLength:
                        if currentMotifID not in motDict.keys():
                            #print 'saving motif %s' % currentMotif
                            motDict[currentMotifID] = currentMotif
                        motifPos = 0
                        currentMotif = ''
            index += 4
            seqLine = self.contents[index]
        
        index = 0
        for key in motDict.keys():
            index += 1
            self.motifs.append(Motif(self.tagID + '-FP-' + str(index), motDict[key]))
            
        return self.motifs
         

                            