###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# locator.py
from cistematic.programs import Program
import os, time, string
from copy import deepcopy
from cistematic.core.motif import Motif

class Locator(Program):
    motifs = []
    consensus = 'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
    identifier = 'DEFAULT'
    PWM = []
    seqs = []
    motFile = ''
    
    def getSettings(self):
        return (self.consensus, self.identifier, self.PWM, self.seqs, self.motFile)
    
    def setSettings(self, settings):
        self.motifs = []
        progsettings = ''
        
        commSettings = 'progsettings = ' + str(settings)
        exec commSettings
        
        (self.consensus, self.identifier, self.PWM, self.seqs, self.motFile) = progsettings
    
    def setGenExpOptions(self, optionArray):
        for option in optionArray:
            (optionTag, optionValue) = option.split(':')
            if optionTag == 'consensus':
                self.setConsensus(optionValue)
            if optionTag == 'identifier':
                self.setID(optionValue)
    
    #set consensus
    def setConsensus(self, cons):
        self.consensus = cons
    
    # set identifier
    def setID(self, ident):
        self.identifier = ident
    
    # set the PWM 
    def setPWM(self, aPWM):
        self.PWM[self.identifier] = aPWM
    
    def useMotifFile(self, mFile):
        self.motFile = mFile

    # run the program - preferrably after some of the other options have been set
    def run(self):
        pass

    def getMotifs(self):
    
        self.motifs = []
        if self.motFile != '':
            self.motifs.append(Motif(self.tagID + '-LOC-' + self.identifier, motifFile = self.motFile))
        else:
            self.motifs.append(Motif(self.tagID + '-LOC-' + self.identifier, self.consensus, self.PWM, self.seqs))
        return self.motifs
