###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Branchistoma floridae
import os, string
from cistematic.genomes import Genome
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'


geneDB = cisRoot + '/B_floridae/bfloridae.genedb' 

def loadChromosomes(db, chromPath, chromOutPath):
    sequenceList = []
    addSequence = 0
    chromosomeList = []
    bfGenome = Genome('bfloridae', dbFile=db)
    
    spacer = 50 * 'N'
    files = os.listdir(chromPath)
    jointseq = ''
    jointseqLen = 0
    index = 0
    for filename in files:
        inFile = open(chromPath + '/' + filename, 'r')
        # skip the header
        header = inFile.readline()
        while header != "":
            seqArray = []
            seqLen = 0
            chromLine = header.strip().split()
            chromID = chromLine[1]
            currentLine = inFile.readline()
            while currentLine != "" and currentLine[0] != ">":
                lineSeq = currentLine.strip()
                seqLen += len(lineSeq)
                seqArray.append(lineSeq)
                currentLine = inFile.readline()
            seq = string.join(seqArray,'')
            jointseq += seq + spacer
            jointseqLen = len(jointseq)
            if jointseqLen > 110000:
                chromID = 'joint%d' % index
                print 'Added contig %s to database' % chromID
                sequenceList.append((('bfloridae', chromID), jointseq, 'chromosome', str(jointseqLen)))
                chromosomeList.append((chromID, chromID, 'db'))
                addSequence += 1
                index += 1
                jointseq = ''
            header = currentLine
            if addSequence > 1000: 
                bfGenome.addSequenceBatch(sequenceList)
                sequenceList = []
                addSequence = 0
        inFile.close()
    if addSequence > 0:
        bfGenome.addSequenceBatch(sequenceList)
    bfGenome.addChromosomeEntryBatch(chromosomeList)

def createDBFile(db):
    bfGenome = Genome('bfloridae',  dbFile=db)
    bfGenome.createGeneDB(db)

def createDBindices(db):
    bfGenome = Genome('bfloridae', dbFile=db)
    bfGenome.createIndices()

def buildBfloridaeDB(db=geneDB):
    chromoPath = cisRoot + '/download/bf'
    chromoOutPath = '/B_floridae/'
        
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Loading sequences" 
    loadChromosomes(db, chromoPath, chromoOutPath)
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db
