###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Gallus gallus
import string
from cistematic.genomes import Genome
from cistematic.core.geneinfo import geneinfoDB
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB = cisRoot + '/G_gallus/chicken.genedb' 

def loadChromosome(db, chromID, chromPath, chromOut):
    seq = ''
    seqArray = []
    seqLen = 0
    ggGenome = Genome('chicken', dbFile=db)
    
    inFile = open(chromPath, 'r')
    # skip the header
    index = 0
    line = inFile.readline()
    for line in inFile:
        seqArray.append(line.strip())
    
    seq = string.join(seqArray,'')
    seqLen = len(seq)
    if seqLen < 1:
        print "Problems reading sequence from file"
    print "writing to file %s" % chromOut
    outFile = open(cisRoot + chromOut,'w')
    outFile.write(seq)
    outFile.close()
    
    #print "calling ggGenome()"
    ggGenome.addChromosomeEntry(chromID, chromOut, 'file')

def loadGeneEntries(db, gFile):
    """ FIXME - NEED TO DEAL WITH ALTERNATIVE SPLICING ENTRIES
    """
    geneEntries = []
    alreadySeen = []
    ggGenome = Genome('chicken', dbFile=db)
    geneFile = open(gFile, 'r')
    
    geneFile.readline()
    for line in geneFile:
        if '|' in line:
            continue
        cols = line.split('\t')
        if cols[11].strip() != 'GENE':
            continue
        
        name = cols[10].split(':')
        gid = name[1]
        if gid == '' or gid in alreadySeen:
            continue
        alreadySeen.append(gid)
        
        start = int(cols[2]) - 1
        stop = int(cols[3]) - 1
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('chicken', gid)
        gidVersion = 1
        geneEntries.append((geneID, chrom, start, stop, sense, 'gene', gidVersion))
    
    print "Adding %d gene entries" % len(geneEntries)
    ggGenome.addGeneEntryBatch(geneEntries)

def loadGeneFeatures(db, gFile):
    """ Load gene features such as CDS, UTR, and PSEUDO from the gene file.
    """
    featureEntries = []
    ggGenome = Genome('chicken', dbFile=db)
    featureFile = open(gFile, 'r')
    
    featureFile.readline()
    for line in featureFile:
        if '|' in line:
            continue
        cols = line.split('\t')
        if cols[11].strip() not in ['CDS', 'UTR', 'PSEUDO']:
            continue
        
        fType = cols[11]
        name = cols[10].split(':')
        gid = name[1]
        if gid == '':
            continue
        
        start = int(cols[2]) - 1
        stop = int(cols[3]) - 1
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('chicken', gid)
        gidVersion = 1
        featureEntries.append((geneID, gidVersion, chrom, start, stop, sense, fType))
    
    print "Adding %d feature entries" % len(featureEntries)
    ggGenome.addFeatureEntryBatch(featureEntries)

def loadGeneOntology(db, goPath, goDefPath):
    ggGenome = Genome('chicken', dbFile=db)
    goDefFile = open(goDefPath, 'r')
    goFile = open(goPath, 'r')
    
    idb = geneinfoDB()
    
    goDefs = {}
    locus = {}
    goArray = []
    
    for goDefEntry in goDefFile:
        if goDefEntry[0] != '!':
            cols = goDefEntry.split('\t')
            goDefs[cols[0]] = (cols[1], cols[2].strip())
    
    goEntries = goFile.readlines()
    
    prevGID = ''
    for entry in goEntries:
        try:
            fields = entry.split('\t')
            if fields[0] != '9031':
                continue
            locID = fields[1].strip()
            gID = ('chicken', locID)
            if prevGID != gID:
                prevGID = gID
                gene_name = ''
                synonyms = idb.geneIDSynonyms(gID)
                if len(synonyms) >0:
                    for entry in synonyms:
                        gene_name += ','    
                        gene_name += entry
                else:
                    gene_name = ' '
            goArray.append((gID, fields[2], '', gene_name[1:], '', string.replace(goDefs[fields[2]][0], "'", "p"), goDefs[fields[2]][1], ''))
        except:
            print "locus ID %s could not be added" % locID
            pass
    
    print "adding %d go entries" % len(goArray)
    ggGenome.addGoInfoBatch(goArray)

def createDBFile(db):
    ggGenome = Genome('chicken',  dbFile=db)
    ggGenome.createGeneDB(db)

def createDBindices(db):
    ggGenome = Genome('chicken', dbFile=db)
    ggGenome.createIndices()

def buildChickenDB(db=geneDB):
    genePath = cisRoot + '/download/seq_gene.md'
    goDefPath = cisRoot + '/download/GO.terms_and_ids' # ftp://ftp.geneontology.org/pub/go/doc/GO.terms_and_ids
    goPath = cisRoot + '/download/gene2go' # ftp://ftp.ncbi.nih.gov/gene/DATA/gene2go.gz
        
    chromos = {'1': cisRoot + '/download/chr1.fa', '2': cisRoot + '/download/chr2.fa', 
    '3': cisRoot + '/download/chr3.fa', '4': cisRoot + '/download/chr4.fa', 
    '5': cisRoot + '/download/chr5.fa', '6': cisRoot + '/download/chr6.fa', 
    '7': cisRoot + '/download/chr7.fa', '8': cisRoot + '/download/chr8.fa', 
    '9': cisRoot + '/download/chr9.fa', '10': cisRoot + '/download/chr10.fa', 
    '11': cisRoot + '/download/chr11.fa', '12': cisRoot + '/download/chr12.fa', 
    '13': cisRoot + '/download/chr13.fa', '14': cisRoot + '/download/chr14.fa', 
    '15': cisRoot + '/download/chr15.fa', '16': cisRoot + '/download/chr16.fa',
    '17': cisRoot + '/download/chr17.fa', '18': cisRoot + '/download/chr18.fa', 
    '19': cisRoot + '/download/chr19.fa', '20': cisRoot + '/download/chr20.fa',  
    '21': cisRoot + '/download/chr21.fa', '22': cisRoot + '/download/chr22.fa',  
    '23': cisRoot + '/download/chr23.fa', '24': cisRoot + '/download/chr24.fa', 
    '25': cisRoot + '/download/chr25.fa', '26': cisRoot + '/download/chr26.fa', 
    '27': cisRoot + '/download/chr27.fa', '28': cisRoot + '/download/chr28.fa', 
    '32': cisRoot + '/download/chr32.fa',  'W': cisRoot + '/download/chrW.fa', 
    'Z': cisRoot + '/download/chrZ.fa', 'M': cisRoot + '/download/chrM.fa', 
    'E22C19W28_E50C23': cisRoot + '/download/chrE22C19W28_E50C23.fa', 'E64': cisRoot + '/download/chrE64.fa',
    '1_random': cisRoot + '/download/chr1_random.fa', '2_random': cisRoot + '/download/chr2_random.fa', 
    '4_random': cisRoot + '/download/chr4_random.fa', '5_random': cisRoot + '/download/chr5_random.fa', 
    '6_random': cisRoot + '/download/chr6_random.fa', '7_random': cisRoot + '/download/chr7_random.fa', 
    '8_random': cisRoot + '/download/chr8_random.fa', '10_random': cisRoot + '/download/chr10_random.fa', 
    '11_random': cisRoot + '/download/chr11_random.fa', '12_random': cisRoot + '/download/chr12_random.fa',
    '13_random': cisRoot + '/download/chr13_random.fa', '16_random': cisRoot + '/download/chr16_random.fa', 
    '17_random': cisRoot + '/download/chr17_random.fa', '18_random': cisRoot + '/download/chr18_random.fa', 
    '20_random': cisRoot + '/download/chr20_random.fa', '22_random': cisRoot + '/download/chr22_random.fa', 
    '25_random': cisRoot + '/download/chr25_random.fa', '28_random': cisRoot + '/download/chr28_random.fa', 
    'Un_random': cisRoot + '/download/chrUn_random.fa',  'W_random': cisRoot + '/download/chrW_random.fa' , 
    'Z_random': cisRoot + '/download/chrZ_random.fa'}
    
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Adding gene entries"
    loadGeneEntries(db, genePath)
    
    #print "Adding gene annotations"
    #loadGeneAnnotations(db, annotPath)
    
    print "Adding gene features"
    loadGeneFeatures(db, genePath)
    
    for chromID in chromos.keys():
        print "Loading chromosome %s" % chromID
        loadChromosome(db, chromID, chromos[chromID], '/G_gallus/chromo' + chromID + '.bin')
    
    print "Adding gene ontology"
    loadGeneOntology(db, goPath, goDefPath)
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db
