###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Pan troglodytes
try:
    from pysqlite2 import dbapi2 as sqlite
except:
    from sqlite3 import dbapi2 as sqlite

from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB = cisRoot + '/P_troglodytes/ANNOTATIONS/chimpanzee.db' 
chromRoot= cisRoot + '/P_troglodytes/CHR_'

chromDict = { '1': '01/chr1', '2': '02/chr2', '3': '03/chr3' , '4': '04/chr4', '5':'05/chr5', 
 '6': '06/chr6', '7': '07/chr7', '8': '08/chr8' , '9': '09/chr9', '10':'10/chr10',
  '11': '11/chr11', '12': '12/chr12', '13': '13/chr13' , '14': '14/chr14', '15':'15/chr15',
   '16': '16/chr16', '17': '17/chr17', '18': '18/chr18' , '19': '19/chr19', '20':'20/chr20',
    '21': '21/chr21', '22': '22/chr22', '23': '23/chr23', 'X':'X/chrX' , 'Y':'Y/chrY', 
    '1rand': '01/chr1_random', '2rand': '02/chr2_random', '3rand': '03/chr3_random' , 
    '4rand': '04/chr4_random', '5rand':'05/chr5_random',  '6rand': '06/chr6_random', 
    '7rand': '07/chr7_random', '8rand': '08/chr8_random' , '9rand': '09/chr9_random', 
    '10rand':'10/chr10_random', '11rand': '11/chr11_random', '12rand': '12/chr12_random',
     '13rand': '13/chr13_random' , '14rand': '14/chr14_random', '15rand':'15/chr15_random', 
     '16rand': '16/chr16_random', '17rand': '17/chr17_random', '18rand': '18/chr18_random' , 
     '19rand': '19/chr19_random',  '20rand': '20/chr20_random', '21rand': '21/chr21_random',
     '22rand': '22/chr22_random',  '23rand':'23/chr23_random', 'Xrand': 'X/chrX_random' , 
     'Yrand':'Y/chrY_random', 'UN':'UN/chrUn_random'
}

def checkGene(geneID):
    """ returns True if the geneID matches an entry in the genome database
    """
    (genome, gID) = geneID
    if genome != 'chimpanzee':
        return False

    db = sqlite.connect(geneDB, timeout=60)
    sql = db.cursor()
    
    stmt = "SELECT chromosome  from gene_entries where name = '%s' " % gID
    sql.execute(stmt)
    
    count = sql.rowcount
    db.close()
    
    if count > 0:
        return True
    
    return False
    
def geneInfo(geneID):
    """ returns (chromosome, start, stop, length, orientation) or '' 
    """
    (genome, gID) = geneID
    result = ""

    db = sqlite.connect(geneDB, timeout=60)
    sql = db.cursor()
    
    stmt = "SELECT chromosome, start, stop, length, orientation from gene_entries where name = '%s'" % gID
    sql.execute(stmt)
    
    if sql.rowcount > 0:
        res = sql.fetchone()
        
        chrom = res[0]
        start = int(res[1])
        stop = int(res[2])
        length = int(res[3])
        orientation = res[4]
        result = (chrom, start, stop, length, orientation)
    
    db.close()
    return result


def chromGeneEntries(chromNum):
    """ returns [list of all orf names]
    """
    result = []

    db = sqlite.connect(geneDB, timeout=60)
    sql = db.cursor()
    
    stmt = "SELECT name, start, stop, orientation from gene_entries where chromosome = '%s'" % chromNum
    sql.execute(stmt)
    res = sql.fetchall()
    db.close()
    
    for entry in res:
        (name, start, stop, sense) = entry
        result.append((int(start), int(stop), sense, ('chimpanzee', name)))
    
    return result

def getAllGenes():
    """ returns [list of all orf names]
    """
    result = []

    db = sqlite.connect(geneDB, timeout=60)
    sql = db.cursor()
    
    stmt = "SELECT name from gene_entries"
    sql.execute(stmt)
    res = sql.fetchall()
    db.close()
    
    for entry in res:
        result.append(entry[0])
    
    return result

def annotInfo(geneID):
    """ returns a list of annotations, if any
    """
    annotations = []

    (genome, gID) = geneID

    db = sqlite.connect(geneDB, timeout=60)
    sql = db.cursor()
    
    stmt = "SELECT description from gene_annotation where name = '%s'" % gID
    sql.execute(stmt)
    
    if sql.rowcount > 0:
        res = sql.fetchall()
        for entry in res:
            annotations.append(entry[0])
        
    db.close()
    
    return annotations

def allAnnotInfo():
    """ returns all annotations in a dictionary of gID: description pairs.
    """
    annotations = {}   
     
    db = sqlite.connect(geneDB, timeout=60)
    sql = db.cursor()
    
    stmt = "SELECT name, description from gene_annotation"
    sql.execute(stmt)
    
    res = sql.fetchall()
    db.close()

    for entry in res:
        annotations[('chimpanzee',entry[0])] = [entry[1]]
    
    return annotations

def allGOInfo():
    """ stub function for now
    """
    annotations = {}
    
    return annotations

def goInfo(geneID):
    """ stub function for now
    """
    annotations = []

    return annotations
