###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Canis familiaris
import string
from cistematic.genomes import Genome
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB = cisRoot + '/C_familiaris/dog.genedb' 

def loadChromosome(db, chromID, chromPath, chromOut):
    seq = ''
    seqArray = []
    seqLen = 0
    cfGenome = Genome('dog', dbFile=db)
    
    inFile = open(chromPath, 'r')
    # skip the header
    index = 0
    line = inFile.readline()
    for line in inFile:
        seqArray.append(line.strip())
    
    seq = string.join(seqArray,'')
    seqLen = len(seq)
    if seqLen < 1:
        print "Problems reading sequence from file"
    print "writing to file %s" % chromOut
    outFile = open(cisRoot + chromOut,'w')
    outFile.write(seq)
    outFile.close()
    
    cfGenome.addChromosomeEntry(chromID, chromOut, 'file')

def loadGeneEntries(db, gFile):
    """ FIXME - NEED TO DEAL WITH ALTERNATIVE SPLICING ENTRIES
    """
    geneEntries = []
    alreadySeen = []
    cfGenome = Genome('dog', dbFile=db)
    geneFile = open(gFile, 'r')
    
    geneFile.readline()
    for line in geneFile:
        cols = line.split('\t')
        if cols[11].strip() != 'GENE':
            continue
        
        name = cols[10].split(':')
        gid = name[1]
        if gid == '' or gid in alreadySeen:
            continue
        alreadySeen.append(gid)
        
        start = int(cols[2]) - 1
        stop = int(cols[3]) - 1
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('dog', gid)
        gidVersion = 1
        geneEntries.append((geneID, chrom, start, stop, sense, 'gene', gidVersion))
    
    print "Adding %d gene entries" % len(geneEntries)
    cfGenome.addGeneEntryBatch(geneEntries)

def loadGeneFeatures(db, gFile):
    """ Load gene features such as CDS, UTR, and PSEUDO from the gene file.
    """
    featureEntries = []
    cfGenome = Genome('dog', dbFile=db)
    featureFile = open(gFile, 'r')
    
    featureFile.readline()
    for line in featureFile:
        cols = line.split('\t')
        if cols[11].strip() not in ['CDS', 'UTR', 'PSEUDO']:
            continue
        
        fType = cols[11]
        name = cols[10].split(':')
        gid = name[1]
        if gid == '':
            continue
        
        start = int(cols[2]) - 1
        stop = int(cols[3]) - 1
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('dog', gid)
        gidVersion = 1
        featureEntries.append((geneID, gidVersion, chrom, start, stop, sense, fType))
    
    print "Adding %d feature entries" % len(featureEntries)
    cfGenome.addFeatureEntryBatch(featureEntries)


def loadGeneAnnotations(db, annotPath):
    geneAnnotations = []
    annotFile = open(annotPath, 'r')  
    locus = {}
    cfGenome = Genome('dog', dbFile=db)
    
    for line in annotFile:
        try:
            cols = line.split('\t')
            locID = cols[0]
            geneName = cols[1]
            geneDesc = cols[6]
            
            if len(locID) > 0:
                geneAnnotations.append((('dog', locID), string.replace(geneDesc.strip(), "'", "p")))
        except:
            pass
    
    print "Adding %d annotations" % len(geneAnnotations)
    cfGenome.addAnnotationBatch(geneAnnotations)

def loadGeneOntology(db, goPath, goDefPath):
    cfGenome = Genome('dog', dbFile=db)
    goDefFile = open(goDefPath, 'r')
    goFile = open(goPath, 'r')
    
    idb = geneinfoDB()
    
    goDefs = {}
    locus = {}
    goArray = []
    
    for goDefEntry in goDefFile:
        if goDefEntry[0] != '!':
            cols = goDefEntry.split('\t')
            goDefs[cols[0]] = (cols[1], cols[2].strip())
    
    goEntries = goFile.readlines()
    
    prevGID = ''
    for entry in goEntries:
        try:
            fields = entry.split('\t')
            if fields[0] != '9615':
                continue
            locID = fields[1].strip()
            gID = ('dog', locID)
            if prevGID != gID:
                prevGID = gID
                gene_name = ''
                synonyms = idb.geneIDSynonyms(gID)
                if len(synonyms) >0:
                    for entry in synonyms:
                        gene_name += ','    
                        gene_name += entry
                else:
                    gene_name = ' '
            goArray.append((gID, fields[2], '', gene_name[1:], '', string.replace(goDefs[fields[2]][0], "'", "p"), goDefs[fields[2]][1], ''))
        except:
            print "locus ID %s could not be added" % locID
            pass
    
    print "adding %d go entries" % len(goArray)
    cfGenome.addGoInfoBatch(goArray)

def createDBFile(db):
    cfGenome = Genome('dog',  dbFile=db)
    cfGenome.createGeneDB(db)

def createDBindices(db):
    cfGenome = Genome('dog', dbFile=db)
    cfGenome.createIndices()

def buildDogDB(db=geneDB):
    genePath = cisRoot + '/download/seq_gene.md'
    
    chromos = {'1': cisRoot + '/download/chr1.fa', '2': cisRoot + '/download/chr2.fa', 
    '3': cisRoot + '/download/chr3.fa', '4': cisRoot + '/download/chr4.fa', 
    '5': cisRoot + '/download/chr5.fa', '6': cisRoot + '/download/chr6.fa', 
    '7': cisRoot + '/download/chr7.fa', '8': cisRoot + '/download/chr8.fa', 
    '9': cisRoot + '/download/chr9.fa', '10': cisRoot + '/download/chr10.fa', 
    '11': cisRoot + '/download/chr11.fa', '12': cisRoot + '/download/chr12.fa', 
    '13': cisRoot + '/download/chr13.fa', '14': cisRoot + '/download/chr14.fa', 
    '15': cisRoot + '/download/chr15.fa', '16': cisRoot + '/download/chr16.fa',
    '17': cisRoot + '/download/chr17.fa', '18': cisRoot + '/download/chr18.fa', 
    '19': cisRoot + '/download/chr19.fa', '20': cisRoot + '/download/chr20.fa',  
    '21': cisRoot + '/download/chr21.fa', '22': cisRoot + '/download/chr22.fa',
    '23': cisRoot + '/download/chr23.fa', '24': cisRoot + '/download/chr24.fa', 
    '25': cisRoot + '/download/chr25.fa', '26': cisRoot + '/download/chr26.fa',
    '27': cisRoot + '/download/chr27.fa', '28': cisRoot + '/download/chr28.fa', 
    '29': cisRoot + '/download/chr29.fa', '30': cisRoot + '/download/chr30.fa',  
    '31': cisRoot + '/download/chr31.fa', '32': cisRoot + '/download/chr32.fa',
    '33': cisRoot + '/download/chr33.fa', '34': cisRoot + '/download/chr34.fa', 
    '35': cisRoot + '/download/chr35.fa', '36': cisRoot + '/download/chr36.fa',
    '37': cisRoot + '/download/chr37.fa', '38': cisRoot + '/download/chr38.fa', 
    'X': cisRoot + '/download/chrX.fa', 'Un': cisRoot + '/download/chrUn.fa'  
    }
    
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Adding gene entries"
    loadGeneEntries(db, genePath)
    
    print "Adding gene features"
    loadGeneFeatures(db, genePath)
    
    for chromID in chromos.keys():
        print "Loading chromosome %s" % chromID
        loadChromosome(db, chromID, chromos[chromID], '/C_familiaris/chromo' + chromID + '.bin')
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db
