###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for ENCODE Regions
import string, os
from cistematic.genomes import Genome
from cistematic.core.geneinfo import geneinfoDB
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB  = cisRoot + '/ENCODE/encode.genedb' 

def loadChromosomes(db, inPath, chromOutPath):
    enGenome = Genome('encode', dbFile=db)
    regionList = os.listdir(inPath)
    for region in regionList:
        regionPath = inPath + '/' + region
        fileList = os.listdir(regionPath)
        for filename in fileList:
            seq = ''
            seqArray = []
            seqLen = 0
            parts = filename.split('.')
            genome = parts[0]
            inFile = open(regionPath + '/' + filename, 'r')
            index = 0
            header = inFile.readline()
            while header != "":
                seqArray = []
                seqLen = 0
                chromID = region + '.' + genome + '-' + str(index)
                currentLine = inFile.readline()
            
                while currentLine != "" and currentLine[0] != ">":
                    lineSeq = currentLine.strip()
                    seqLen += len(lineSeq)
                    seqArray.append(lineSeq)
                    currentLine = inFile.readline()
                
                seq = string.join(seqArray,'')
                if seqLen < 10000:
                    print 'Added contig %s to database' % chromID
                    enGenome.addSequence(('encode', chromID), seq, 'chromosome', str(seqLen))
                    enGenome.addChromosomeEntry(chromID, chromID, 'db')
                else:
                    outFileName = chromOutPath + '/' + chromID + '.bin'
                    outFile = open(cisRoot + outFileName, 'w')
                    outFile.write(seq)
                    outFile.close()
                    print 'Added contig file %s to database' % outFileName
                    enGenome.addChromosomeEntry(chromID, outFileName, 'file')
                index += 1
                header = currentLine
            inFile.close()


def fixRandoms(db, gfile, afile, efile):
    geneFile = open(gfile,'r')
    annotFile = open(afile,'r')
    encodeFile = open(efile, 'r')
    
    senseArray = {'+': 'F', '-': 'R', '.':'F'}
    genesToFix = {}
    seenArray = []
    insertArray = []
    startDict = {}
    stopDict = {}
    ref2gene = {}
    encodeChr = []
    chrToRegion = {}
    encodeRegions = {}
    
    for line in efile:
        fields = line.split('\t')
        enchr = fields[0]
        enstart = fields[1]
        enstop = fields[2]
        region = fields[3]
        if enchr not in encodeChr:
            encodeChr.append(enchr)
            chrToRegion[enchr] = []
        chrToRegion.append(region)
        newChr = region + '.human-0'
        encodeRegions[region] = (start, stop, newChr)
    
    for annotLine in annotFile:
        annotFields = annotLine.split('\t')
        refseq = annotFields[2]
        geneid = annotFields[6]
        ref2gene[refseq] = geneid
        
    for geneLine in geneFile:
        geneFields = geneLine.split('\t')
        chrom = geneFields[1]
        if chrom not in encodeChr:
            continue
        
        exonNum = int(geneFields[7])
        exonStarts = geneFields[8].split(',')
        exonStops = geneFields[9].split(',')
        sense = senseArray[geneFields[2]]
        gstop = int(geneFields[6]) - 1
        gstart = int(geneFields[5]) - 1
        refseq = geneFields[0]
        try:
            geneID = ('encode', ref2gene[refseq])
        except:
            continue
        
        gidVersion = '1'
        if geneID in seenArray:
            gidVersion = '2' # doesn't deal with more than 2 refseq's for the same locus, yet.
        else:
            seenArray.append(geneID)
        
        insertArray.append((geneID, encodeChrom, enStart, enStop, sense, 'CDS', gidVersion))

    
    print 'Adding %d entries' % len(insertArray)
    enGenome = Genome('encode', dbFile=db)
    enGenome.writeBatchDB(updateArray)


def loadGeneFeatures2(db, gfile, afile):
    geneFile = open(gfile,'r')
    annotFile = open(afile,'r')
    
    senseArray = {'+': 'F', '-': 'R', '.':'F'}
    genesToFix = {}
    seenArray = []
    insertArray = []
    startDict = {}
    stopDict = {}
    ref2gene = {}
    
    for annotLine in annotFile:
        annotFields = annotLine.split('\t')
        refseq = annotFields[2]
        geneid = annotFields[6]
        ref2gene[refseq] = geneid
        
    for geneLine in geneFile:
        geneFields = geneLine.split('\t')
        exonNum = int(geneFields[7])
        exonStarts = geneFields[8].split(',')
        exonStops = geneFields[9].split(',')
        chrom = geneFields[1][3:]
        sense = senseArray[geneFields[2]]
        gstop = int(geneFields[6]) - 1
        gstart = int(geneFields[5]) - 1
        refseq = geneFields[0]
        try:
            geneID = ('encode', ref2gene[refseq])
        except:
            continue
        
        gidVersion = '1'
        if geneID in seenArray:
            gidVersion = '2' # doesn't deal with more than 2 refseq's for the same locus, yet.
        else:
            seenArray.append(geneID)
        
        for index in range(exonNum):
            estart = int(exonStarts[index]) - 1
            estop = int(exonStops[index]) - 1
            if estart >= gstart and estop <= gstop:
                insertArray.append((geneID, gidVersion, chrom, estart, estop, sense, 'CDS'))
            elif estop <= gstart:
                if sense == 'F':
                    fType = '5UTR'
                else:
                    fType = '3UTR'
                insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
            elif estart >= gstop:
                if sense == 'F':
                    fType = '3UTR'
                else:
                    fType = '5UTR'
                insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
            elif estart <= gstop:
                if sense == 'F':
                    fType = '3UTR'
                else:
                    fType = '5UTR'
                insertArray.append((geneID, 1, chrom, estart, gstop, sense, 'CDS'))
                insertArray.append((geneID, 1, chrom, gstop + 1, estop, sense, fType))
            else:
                if sense == 'F':
                    fType = '5UTR'
                else:
                    fType = '3UTR'
                insertArray.append((geneID, 1, chrom, gstart, estop, sense, 'CDS'))
                insertArray.append((geneID, 1, chrom, estart, gstart - 1, sense, fType))
    geneFile.close()
    
    enGenome = Genome('encode', dbFile=db)
    print 'Adding %d features' % len(insertArray)
    enGenome.addFeatureEntryBatch(insertArray)

def loadGeneAnnotations(db):
    geneAnnotations = []
    idb = geneinfoDB()  
    gidList = []
    mmGenome = Genome('mouse', dbFile=db)
    
    gidList = mmGenome.allGIDs()
    for locID in gidList:
        gID = ('mouse', locID)
        geneDescArray = idb.getDescription(gID)
        geneDesc = ''
        for entry in geneDescArray:
            geneDesc += ','
            geneDesc += entry.strip()
        if len(geneDescArray) > 0:
            geneAnnotations.append((gID, string.replace(geneDesc[1:], "'", "p")))
    
    print "Adding %d annotations" % len(geneAnnotations)
    mmGenome.addAnnotationBatch(geneAnnotations)

def createDBFile(db):
    enGenome = Genome('encode',  dbFile=db)
    enGenome.createGeneDB(db)

def createDBindices(db):
    enGenome = Genome('encode', dbFile=db)
    enGenome.createIndices()

def buildEncodeDB(db=geneDB):
    refGene=cisRoot + '/download/ENCODE/refGene.txt'
    refLink=cisRoot + '/download/ENCODE/refLink.txt'
    enRegions = cisRoot + '/download/ENCODE/encodeRegions.txt'
    
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Adding gene entries"
    loadGeneEntries(db, refGene, refLink, enRegions)
    
    print "Adding gene features"
    #loadGeneFeatures2(db, cisRoot + '/download/refGene.txt', cisRoot + '/download/refLink.txt')
    
    print "Adding gene annotations"
    #loadGeneAnnotations(db)
    
    print "Loading Chromosomes"
    loadChromosomes(db, cisRoot + '/download/ENCODE/MAY-2005' , '/ENCODE')
    
    print "Creating Indices"
    #createDBindices(db)
    
    print "Finished creating database %s" % db


    