###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Homo sapiens
import string
from cistematic.genomes import Genome
from cistematic.core.geneinfo import geneinfoDB
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

# default annotations
geneDB = cisRoot + '/H_sapiens/human.genedb' 


def loadChromosome(db, chromID, chromPath, chromOut):
    seq = ''
    seqArray = []
    seqLen = 0
    hsGenome = Genome('human', dbFile=db)
    
    inFile = open(chromPath, 'r')
    # skip the header
    index = 0
    line = inFile.readline()
    for line in inFile:
        seqArray.append(line.strip())
    
    seq = string.join(seqArray,'')
    seqLen = len(seq)
    if seqLen < 1:
        print "Problems reading sequence from file"
    print "writing to file %s" % chromOut
    outFile = open(cisRoot + chromOut,'w')
    outFile.write(seq)
    outFile.close()
    
    #print "calling hsGenome()"
    hsGenome.addChromosomeEntry(chromID, chromOut, 'file')

def loadGeneEntries(db, gFile):
    """ FIXME - NEED TO DEAL WITH ALTERNATIVE SPLICING ENTRIES
    """
    geneEntries = []
    hsGenome = Genome('human', dbFile=db)
    geneFile = open(gFile, 'r')
    
    for line in geneFile:
        cols = line.split('\t')
        if cols[11] != 'GENE':
            continue
        if cols[12] == 'Celera':
            continue
        
        name = cols[10].split(':')
        gid = name[1]
        start = int(cols[2])
        stop = int(cols[3])
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('human', gid)
        gidVersion = 1
        geneEntries.append((geneID, chrom, start, stop, sense, 'gene', gidVersion))
    
    print "Adding %d gene entries" % len(geneEntries)
    hsGenome.addGeneEntryBatch(geneEntries)

def loadGeneFeatures(db, gFile):
    """ Load gene features such as CDS, UTR, and PSEUDO from the gene file.
    """
    featureEntries = []
    hsGenome = Genome('human', dbFile=db)
    featureFile = open(gFile, 'r')
    
    for line in featureFile:
        cols = line.split('\t')
        if cols[11] not in ['CDS', 'UTR', 'PSEUDO']:
            continue
        if cols[12] == 'Celera':
            continue
        
        fType = cols[11]
        name = cols[10].split(':')
        gid = name[1]
        start = int(cols[2])
        stop = int(cols[3])
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('human', gid)
        gidVersion = 1
        featureEntries.append((geneID, gidVersion, chrom, start, stop, sense, fType))
    
    print "Adding %d feature entries" % len(featureEntries)
    hsGenome.addFeatureEntryBatch(featureEntries)

def loadGeneAnnotations(db):
    geneAnnotations = []
    idb = geneinfoDB()  
    gidList = []
    hsGenome = Genome('human', dbFile=db)
    
    gidList = hsGenome.allGIDs()
    for locID in gidList:
        gID = ('human', locID)
        geneDescArray = idb.getDescription(gID)
        geneDesc = ''
        for entry in geneDescArray:
            geneDesc += ','
            geneDesc += entry.strip()
        if len(geneDescArray) > 0:
            geneAnnotations.append((gID, string.replace(geneDesc[1:], "'", "p")))
    
    print "Adding %d annotations" % len(geneAnnotations)
    hsGenome.addAnnotationBatch(geneAnnotations)

def loadGeneOntology(db, goPath, goDefPath):
    hsGenome = Genome('human', dbFile=db)
    goDefFile = open(goDefPath, 'r')
    goFile = open(goPath, 'r')
    
    idb = geneinfoDB()
    
    goDefs = {}
    locus = {}
    goArray = []
    
    for goDefEntry in goDefFile:
        if goDefEntry[0] != '!':
            cols = goDefEntry.split('\t')
            goDefs[cols[0]] = (cols[1], cols[2].strip())
    
    prevGID = ''
    index = 0
    for entry in goFile:
        try:
            fields = entry.split('\t')
            if fields[0] != '9606':
                continue
            index += 1
            if index % 1000 == 0:
                print 'adding 1000 go entries'
                hsGenome.addGoInfoBatch(goArray)
                goArray = []
            locID = fields[1].strip()
            gID = ('human', locID)
            if prevGID != gID:
                prevGID = gID
                gene_name = ''
                synonyms = idb.geneIDSynonyms(gID)
                if len(synonyms) >0:
                    for entry in synonyms:
                        gene_name += ','    
                        gene_name += entry
                else:
                    gene_name = ' '
            goArray.append((gID, fields[2], '', gene_name[1:], '', string.replace(goDefs[fields[2]][0], "'", "p"), goDefs[fields[2]][1], ''))
        except:
            print "locus ID %s could not be added" % locID
            pass
    
    print "adding %d go entries" % len(goArray)
    hsGenome.addGoInfoBatch(goArray)

def createDBFile(db):
    hsGenome = Genome('human',  dbFile=db)
    hsGenome.createGeneDB(db)

def createDBindices(db):
    hsGenome = Genome('human', dbFile=db)
    hsGenome.createIndices()

def buildHumanDB(db=geneDB, downloadDir= cisRoot + '/download'):
    genePath = downloadDir + '/seq_gene.md'
    goDefPath = downloadDir + '/GO.terms_and_ids'
    goPath = downloadDir + '/gene2go'
    
    chromos = {'1': downloadDir + '/chr1.fa', '2': downloadDir + '/chr2.fa', '3': downloadDir + '/chr3.fa', 
    '4': downloadDir + '/chr4.fa', '5': downloadDir + '/chr5.fa', '6': downloadDir + '/chr6.fa', 
    '7': downloadDir + '/chr7.fa', '8': downloadDir + '/chr8.fa', '9': downloadDir + '/chr9.fa', 
    '10': downloadDir + '/chr10.fa', '11': downloadDir + '/chr11.fa', '12': downloadDir + '/chr12.fa', 
    '13': downloadDir + '/chr13.fa', '14': downloadDir + '/chr14.fa', '15': downloadDir + '/chr15.fa', 
    '16': downloadDir + '/chr16.fa', '17': downloadDir + '/chr17.fa', '18': downloadDir + '/chr18.fa', 
    '19': downloadDir + '/chr19.fa', '20': downloadDir + '/chr20.fa', '21': downloadDir + '/chr21.fa', 
    '22': downloadDir + '/chr22.fa', 'X': downloadDir + '/chrX.fa', 'Y': downloadDir + '/chrY.fa', 
    '1rand': downloadDir + '/chr1_random.fa', '2rand': downloadDir + '/chr2_random.fa', 
    '3rand': downloadDir + '/chr3_random.fa', '4rand': downloadDir + '/chr4_random.fa', 
    '5rand': downloadDir + '/chr5_random.fa', '6rand': downloadDir + '/chr6_random.fa', 
    '7rand': downloadDir + '/chr7_random.fa', '8rand': downloadDir + '/chr8_random.fa', 
    '9rand': downloadDir + '/chr9_random.fa', '10rand': downloadDir + '/chr10_random.fa', 
    '11rand': downloadDir + '/chr11_random.fa','13rand': downloadDir + '/chr13_random.fa', 
    '15rand': downloadDir + '/chr15_random.fa', '16rand': downloadDir + '/chr16_random.fa', 
    '17rand': downloadDir + '/chr17_random.fa', '18rand': downloadDir + '/chr18_random.fa', 
    '19rand': downloadDir + '/chr19_random.fa', '21rand': downloadDir + '/chr21_random.fa', 
    '22rand': downloadDir + '/chr22_random.fa', 'Xrand': downloadDir + '/chrX_random.fa'}
    
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Adding gene entries"
    loadGeneEntries(db, genePath)
    
    print "Adding gene features"
    loadGeneFeatures(db, genePath)
    
    print "Adding gene annotations"
    loadGeneAnnotations(db)
    
    print "Adding gene ontology"
    loadGeneOntology(db, goPath, goDefPath)
    
    for chromID in chromos.keys():
        print "Loading chromosome %s" % chromID
        loadChromosome(db, chromID, chromos[chromID], '/H_sapiens/chromo' + chromID + '.bin')
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db
