###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Mus musculus
import string
from cistematic.genomes import Genome
from cistematic.core.geneinfo import geneinfoDB
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB  = cisRoot + '/M_musculus/mouse.genedb' 

def loadChromosome(db, chromID, chromPath, chromOut):
    seq = ''
    seqArray = []
    seqLen = 0
    mmGenome = Genome('mouse', dbFile=db)
    
    inFile = open(chromPath, 'r')
    # skip the header
    index = 0
    line = inFile.readline()
    for line in inFile:
        seqArray.append(line.strip())
    
    seq = string.join(seqArray,'')
    seqLen = len(seq)
    if seqLen < 1:
        print "Problems reading sequence from file"
    print "writing to file %s" % chromOut
    outFile = open(cisRoot + chromOut,'w')
    outFile.write(seq)
    outFile.close()
    
    #print "calling mmGenome()"
    mmGenome.addChromosomeEntry(chromID, chromOut, 'file')

def loadGeneEntries(db, gFile):
    """ FIXME - NEED TO DEAL WITH ALTERNATIVE SPLICING ENTRIES
    """
    geneEntries = []
    alreadySeen = []
    mmGenome = Genome('mouse', dbFile=db)
    geneFile = open(gFile, 'r')
    
    for line in geneFile:
        cols = line.split('\t')
        if cols[11].strip() != 'GENE' or cols[12] != 'C57BL/6J':
            continue
        
        name = cols[10].split(':')
        gid = name[1]
        if gid == '' or gid in alreadySeen:
            continue
        alreadySeen.append(gid)
        
        start = int(cols[2]) - 1
        stop = int(cols[3]) - 1
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('mouse', gid)
        gidVersion = 1
        geneEntries.append((geneID, chrom, start, stop, sense, 'gene', gidVersion))
    
    print "Adding %d gene entries" % len(geneEntries)
    mmGenome.addGeneEntryBatch(geneEntries)

def loadGeneFeatures(db, gFile):
    """ Load gene features such as CDS, UTR, and PSEUDO from the gene file.
    """
    featureEntries = []
    
    mmGenome = Genome('mouse', dbFile=db)
    featureFile = open(gFile, 'r')
    
    for line in featureFile:
        cols = line.split('\t')
        if cols[11].strip() not in ['CDS', 'UTR', 'PSEUDO'] or cols[12] != 'C57BL/6J':
            continue
        
        fType = cols[11]
        name = cols[10].split(':')
        gid = name[1]
        if gid == '':
            continue
        
        start = int(cols[2]) - 1
        stop = int(cols[3]) - 1
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('mouse', gid)
        gidVersion = 1
        featureEntries.append((geneID, gidVersion, chrom, start, stop, sense, fType))
    
    print "Adding %d feature entries" % len(featureEntries)
    mmGenome.addFeatureEntryBatch(featureEntries)

def loadGeneFeatures2(db, gfile, afile):
    geneFile = open(gfile,'r')
    annotFile = open(afile,'r')
    
    senseArray = {'+': 'F', '-': 'R', '.':'F'}
    genesToFix = {}
    seenArray = []
    insertArray = []
    startDict = {}
    stopDict = {}
    ref2gene = {}
    
    for annotLine in annotFile:
        annotFields = annotLine.split('\t')
        refseq = annotFields[2]
        geneid = annotFields[6]
        ref2gene[refseq] = geneid
        
    for geneLine in geneFile:
        geneFields = geneLine.split('\t')
        exonNum = int(geneFields[7])
        exonStarts = geneFields[8].split(',')
        exonStops = geneFields[9].split(',')
        chrom = geneFields[1][3:]
        sense = senseArray[geneFields[2]]
        gstop = int(geneFields[6]) - 1
        gstart = int(geneFields[5]) - 1
        refseq = geneFields[0]
        try:
            geneID = ('mouse', ref2gene[refseq])
        except:
            continue
        
        gidVersion = '1'
        if geneID in seenArray:
            gidVersion = '2' # doesn't deal with more than 2 refseq's for the same locus, yet.
        else:
            seenArray.append(geneID)
        
        for index in range(exonNum):
            estart = int(exonStarts[index]) - 1
            estop = int(exonStops[index]) - 1
            if estart >= gstart and estop <= gstop:
                insertArray.append((geneID, gidVersion, chrom, estart, estop, sense, 'CDS'))
            elif estop <= gstart:
                if sense == 'F':
                    fType = '5UTR'
                else:
                    fType = '3UTR'
                insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
            elif estart >= gstop:
                if sense == 'F':
                    fType = '3UTR'
                else:
                    fType = '5UTR'
                insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
            elif estart <= gstop:
                if sense == 'F':
                    fType = '3UTR'
                else:
                    fType = '5UTR'
                insertArray.append((geneID, 1, chrom, estart, gstop, sense, 'CDS'))
                insertArray.append((geneID, 1, chrom, gstop + 1, estop, sense, fType))
            else:
                if sense == 'F':
                    fType = '5UTR'
                else:
                    fType = '3UTR'
                insertArray.append((geneID, 1, chrom, gstart, estop, sense, 'CDS'))
                insertArray.append((geneID, 1, chrom, estart, gstart - 1, sense, fType))
    geneFile.close()
    
    mmGenome = Genome('mouse', dbFile=db)
    print 'Adding %d features' % len(insertArray)
    mmGenome.addFeatureEntryBatch(insertArray)

def loadGeneAnnotations(db):
    geneAnnotations = []
    idb = geneinfoDB()  
    gidList = []
    mmGenome = Genome('mouse', dbFile=db)
    
    gidList = mmGenome.allGIDs()
    for locID in gidList:
        gID = ('mouse', locID)
        geneDescArray = idb.getDescription(gID)
        geneDesc = ''
        for entry in geneDescArray:
            geneDesc += ','
            geneDesc += entry.strip()
        if len(geneDescArray) > 0:
            geneAnnotations.append((gID, string.replace(geneDesc[1:], "'", "p")))
    
    print "Adding %d annotations" % len(geneAnnotations)
    mmGenome.addAnnotationBatch(geneAnnotations)

def loadGeneOntology(db, goPath, goDefPath):
    mmGenome = Genome('mouse', dbFile=db)
    goDefFile = open(goDefPath, 'r')
    goFile = open(goPath, 'r')
    
    idb = geneinfoDB()
    
    goDefs = {}
    locus = {}
    goArray = []
    
    for goDefEntry in goDefFile:
        if goDefEntry[0] != '!':
            cols = goDefEntry.split('\t')
            goDefs[cols[0]] = (cols[1], cols[2].strip())
    
    goEntries = goFile.readlines()
    
    prevGID = ''
    for entry in goEntries:
        try:
            fields = entry.split('\t')
            if fields[0] != '10090':
                continue
            locID = fields[1].strip()
            gID = ('mouse', locID)
            if prevGID != gID:
                prevGID = gID
                gene_name = ''
                synonyms = idb.geneIDSynonyms(gID)
                if len(synonyms) >0:
                    for entry in synonyms:
                        gene_name += ','    
                        gene_name += entry
                else:
                    gene_name = ' '
            goArray.append((gID, fields[2], '', gene_name[1:], '', string.replace(goDefs[fields[2]][0], "'", "p"), goDefs[fields[2]][1], ''))
        except:
            print "locus ID %s could not be added" % locID
            pass
    
    print "adding %d go entries" % len(goArray)
    mmGenome.addGoInfoBatch(goArray)

def createDBFile(db):
    mmGenome = Genome('mouse',  dbFile=db)
    mmGenome.createGeneDB(db)

def createDBindices(db):
    mmGenome = Genome('mouse', dbFile=db)
    mmGenome.createIndices()

def buildMouseDB(db=geneDB, downloadDir= cisRoot + '/download'):
    genePath = downloadDir + '/seq_gene.md' # ftp://ftp.ncbi.nih.gov/genomes/M_musculus/mapview/seq_gene.md
    goDefPath = downloadDir + '/GO.terms_and_ids' # ftp://ftp.geneontology.org/pub/go/doc/GO.terms_and_ids
    goPath = downloadDir + '/gene2go' # ftp://ftp.ncbi.nih.gov/gene/DATA/gene2go.gz
    
    # chromosomes are from ftp://hgdownload.cse.ucsc.edu/goldenPath/mm9/chromosomes
    chromos = {'1': downloadDir + '/chr1.fa', '2': downloadDir + '/chr2.fa', 
    '3': downloadDir + '/chr3.fa', '4': downloadDir + '/chr4.fa', 
    '5': downloadDir + '/chr5.fa', '6': downloadDir + '/chr6.fa', 
    '7': downloadDir + '/chr7.fa', '8': downloadDir + '/chr8.fa', 
    '9': downloadDir + '/chr9.fa', '10': downloadDir + '/chr10.fa', 
    '11': downloadDir + '/chr11.fa', '12': downloadDir + '/chr12.fa', 
    '13': downloadDir + '/chr13.fa', '14': downloadDir + '/chr14.fa', 
    '15': downloadDir + '/chr15.fa', '16': downloadDir + '/chr16.fa',
    '17': downloadDir + '/chr17.fa', '18': downloadDir + '/chr18.fa', 
    '19': downloadDir + '/chr19.fa', 'Un': downloadDir + '/chrUn_random.fa',  
    'X': downloadDir + '/chrX.fa', 'Y': downloadDir + '/chrY.fa',
    '1rand': downloadDir + '/chr1_random.fa', '3rand': downloadDir + '/chr3_random.fa',  
    '4rand': downloadDir + '/chr4_random.fa', '5rand': downloadDir + '/chr5_random.fa',  
    '7rand': downloadDir + '/chr7_random.fa', '8rand': downloadDir + '/chr8_random.fa', 
    '9rand': downloadDir + '/chr9_random.fa', 
    '13rand': downloadDir + '/chr13_random.fa', '16rand': downloadDir + '/chr16_random.fa', 
    '17rand': downloadDir + '/chr17_random.fa', 'Xrand': downloadDir + '/chrX_random.fa',  
    'Yrand': downloadDir + '/chrY_random.fa', 'M': downloadDir + '/chrM.fa'}
    
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Adding gene entries"
    loadGeneEntries(db, genePath)
    
    #print "Fixing genes on random chromosomes"
    #fixRandoms()
    
    print "Adding gene features"
    loadGeneFeatures(db, genePath)
    
    #print "Adding Gene Features #2"
    #loadGeneFeatures2(db, cisRoot + '/download/refGene.txt', cisRoot + '/download/refLink.txt')
    
    #print "Fixing features on random chromosomes"
    #fixRandomFeatures()
    
    print "Adding gene annotations"
    loadGeneAnnotations(db)
    
    print "Adding gene ontology"
    loadGeneOntology(db, goPath, goDefPath)
    
    for chromID in chromos.keys():
        print "Loading chromosome %s" % chromID
        loadChromosome(db, chromID, chromos[chromID], '/M_musculus/chromo' + chromID + '.bin')
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db

def fixRandoms(refGene=cisRoot + '/download/refGene.txt', refLink=cisRoot + '/download/refLink.txt', db=geneDB):
    geneFile = open(refGene,'r')
    linkFile = open(refLink, 'r')
    
    randChrom = { 'chrUn_random': 'Un', 'chr1_random': '1rand', 'chr2_random': '2rand','chr3_random': '3rand',
    'chr4_random': '4rand','chr5_random': '5rand','chr6_random': '6rand','chr7_random': '7rand','chr8_random': '8rand',
    'chr9_random': '9rand','chr10_random': '10rand','chr11_random': '11rand','chr12_random': '12rand','chr13_random': '13rand',
    'chr14_random': '14rand','chr15_random': '15rand','chr16_random': '16rand','chr17_random': '17rand','chr18_random': '18rand',
    'chr19_random': '19rand'}
    senseArray = {'+': 'F', '-': 'R'}
    genesToFix = {}
    updateArray = []
    
    for line in geneFile:
        fields = line.split('\t')
        if fields[1] in randChrom:
            genesToFix[fields[0]] = (randChrom[fields[1]], senseArray[fields[2]], fields[3], fields[4])
    
    geneFile.close()
    
    stmt = "UPDATE gene_entries set chromosome = ?, start = ?, stop=?, orientation=? where name = ? " 
    for line in linkFile:
        fields = line.split('\t')
        if fields[2] in genesToFix:
            (chrom, sense, start, stop) = genesToFix[fields[2]]
            updateArray.append((chrom, str(int(start) - 1),str(int(stop) - 1), sense, fields[6]))
    
    linkFile.close()
    
    print 'Fixing %d entries' % len(updateArray)
    mmGenome = Genome('mouse', dbFile=db)
    mmGenome.writeBatchDB(stmt, updateArray)

def fixRandomFeatures(refGene=cisRoot + '/download/refGene.txt', refLink=cisRoot + '/download/refLink.txt', db=geneDB):
    geneFile = open(refGene,'r')
    linkFile = open(refLink, 'r')
    
    randChrom = { 'chrUn_random': 'Un', 'chr1_random': '1rand', 'chr2_random': '2rand','chr3_random': '3rand',
    'chr4_random': '4rand','chr5_random': '5rand','chr6_random': '6rand','chr7_random': '7rand','chr8_random': '8rand',
    'chr9_random': '9rand','chr10_random': '10rand','chr11_random': '11rand','chr12_random': '12rand','chr13_random': '13rand',
    'chr14_random': '14rand','chr15_random': '15rand','chr16_random': '16rand','chr17_random': '17rand','chr18_random': '18rand',
    'chr19_random': '19rand'}
    senseArray = {'+': 'F', '-': 'R'}
    genesToFix = {}
    deleteArray = []
    insertArray = []
    startDict = {}
    stopDict = {}
    
    for line in geneFile:
        fields = line.split('\t')
        if fields[1] in randChrom:
            genesToFix[fields[0]] = line
            startDict[fields[0]] = int(fields[5])
            stopDict[fields[0]] = int(fields[6])
    
    geneFile.close()
    
    stmt = "Delete from sequence_features where name = ? "
    for line in linkFile:
        fields = line.split('\t')
        if fields[2] in genesToFix:
            deleteArray.append([fields[6]])
            geneLine = genesToFix[fields[2]]
            geneFields = geneLine.split('\t')
            exonNum = int(geneFields[7])
            exonStarts = geneFields[8].split(',')
            exonStops = geneFields[9].split(',')
            chrom = randChrom[geneFields[1]]
            sense = senseArray[geneFields[2]]
            gstop = stopDict[fields[2]] - 1
            gstart = startDict[fields[2]] - 1
            geneID = ('mouse', fields[6])
            for index in range(exonNum):
                estart = int(exonStarts[index]) - 1
                estop = int(exonStops[index]) - 1
                if estart >= gstart and estop <= gstop:
                    insertArray.append((geneID, 1, chrom, estart, estop, sense, 'CDS'))
                elif estop <= gstart:
                    if sense == 'F':
                        fType = '5UTR'
                    else:
                        fType = '3UTR'
                    insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
                elif estart >= gstop:
                    if sense == 'F':
                        fType = '3UTR'
                    else:
                        fType = '5UTR'
                    insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
                elif estart <= gstop:
                    if sense == 'F':
                        fType = '3UTR'
                    else:
                        fType = '5UTR'
                    insertArray.append((geneID, 1, chrom, estart, gstop, sense, 'CDS'))
                    insertArray.append((geneID, 1, chrom, gstop + 1, estop, sense, fType))
                else:
                    if sense == 'F':
                        fType = '5UTR'
                    else:
                        fType = '3UTR'
                    insertArray.append((geneID, 1, chrom, gstart, estop, sense, 'CDS'))
                    insertArray.append((geneID, 1, chrom, estart, gstart - 1, sense, fType))

    
    linkFile.close()
    
    print 'Fixing %d genes' % len(deleteArray)
    mmGenome = Genome('mouse', dbFile=db)
    mmGenome.writeBatchDB(stmt, deleteArray)
    print 'Adding %d features' % len(insertArray)
    mmGenome.addFeatureEntryBatch(insertArray)

    