# data for Mus musculus
import sqlite
import string
from cistematic.genomes import Genome

geneDB  = '/proj/genome/M_musculus/mouse.genedb' 


def loadChromosome(db, chromID, chromPath, chromOut):
    seq = ''
    seqArray = []
    seqLen = 0
    mmGenome = Genome('mouse', dbFile=db)
    
    inFile = open(chromPath, 'r')
    # skip the header
    index = 0
    line = inFile.readline()
    for line in inFile:
        seqArray.append(line.strip())
    
    seq = string.join(seqArray,'')
    seqLen = len(seq)
    if seqLen < 1:
        print "Problems reading sequence from file"
    print "writing to file %s" % chromOut
    outFile = open(chromOut,'w')
    outFile.write(seq)
    outFile.close()
    
    #print "calling mmGenome()"
    mmGenome.addChromosomeEntry(chromID, chromOut, 'file')

def loadGeneEntries(db, gFile):
    """ FIXME - NEED TO DEAL WITH ALTERNATIVE SPLICING ENTRIES
    """
    geneEntries = []
    mmGenome = Genome('mouse', dbFile=db)
    geneFile = open(gFile, 'r')
    
    for line in geneFile:
        cols = line.split('\t')
        if cols[11] != 'GENE':
            continue
        if cols[12] == 'Celera':
            continue
        
        name = cols[10].split(':')
        gid = name[1]
        start = int(cols[2]) - 1
        stop = int(cols[3]) - 1
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('mouse', gid)
        gidVersion = 1
        geneEntries.append((geneID, chrom, start, stop, sense, 'gene', gidVersion))
    
    print "Adding %d gene entries" % len(geneEntries)
    mmGenome.addGeneEntryBatch(geneEntries)

def loadGeneEntries(db, gFile, afile):
    geneEntries = []
    dmGenome = Genome('mouse', dbFile=db)
    geneFile = open(gFile, 'r')
    annotFile = open(afile,'r')
    ref2gene = {}
    seenArray = []
    
    for annotLine in annotFile:
        annotFields = annotLine.split('\t')
        refseq = annotFields[2]
        geneid = annotFields[6]
        ref2gene[refseq] = geneid
    
    for line in geneFile:
        cols = line.split('\t')
        
        refseq = cols[0]
        try:
            gid = ref2gene[refseq]
        except:
            continue
        start = int(cols[5])
        stop = int(cols[6])
        sense = cols[2]
        chrom = cols[1][3:]
        if sense == '-':
            sense = 'R'
        else:
            sense = 'F'
        
        geneID = ('mouse', gid)
        
        gidVersion = '1'
        if geneID in seenArray:
            gidVersion = '2' # doesn't deal with more that 2 refseq/gene gracefully.
        else:
            seenArray.append(geneID)
        
        geneEntries.append((geneID, chrom, start, stop, sense, 'gene', gidVersion))
    
    print "Adding %d gene entries" % len(geneEntries)
    dmGenome.addGeneEntryBatch(geneEntries)

def loadGeneFeatures(db, gfile, afile):
    geneFile = open(gfile,'r')
    annotFile = open(afile,'r')
    
    senseArray = {'+': 'F', '-': 'R', '.':'F'}
    genesToFix = {}
    seenArray = []
    insertArray = []
    startDict = {}
    stopDict = {}
    ref2gene = {}
    
    for annotLine in annotFile:
        annotFields = annotLine.split('\t')
        refseq = annotFields[2]
        geneid = annotFields[6]
        ref2gene[refseq] = geneid
        
    for geneLine in geneFile:
        geneFields = geneLine.split('\t')
        exonNum = int(geneFields[7])
        exonStarts = geneFields[8].split(',')
        exonStops = geneFields[9].split(',')
        chrom = geneFields[1][3:]
        sense = senseArray[geneFields[2]]
        gstop = int(geneFields[5]) - 1
        gstart = int(geneFields[6]) - 1
        refseq = geneFields[0]
        try:
            geneID = ('mouse', ref2gene[refseq])
        except:
            continue
        
        gidVersion = '1'
        if geneID in seenArray:
            gidVersion = '2' # doesn't deal with more than 2 refseq's for the same locus, yet.
        else:
            seenArray.append(geneID)
        
        for index in range(exonNum):
            estart = int(exonStarts[index]) - 1
            estop = int(exonStops[index]) - 1
            if estart >= gstart and estop <= gstop:
                insertArray.append((geneID, gidVersion, chrom, estart, estop, sense, 'CDS'))
            elif estop <= gstart:
                if sense == 'F':
                    fType = '5UTR'
                else:
                    fType = '3UTR'
                insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
            elif estart >= gstop:
                if sense == 'F':
                    fType = '3UTR'
                else:
                    fType = '5UTR'
                insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
            elif estart <= gstop:
                if sense == 'F':
                    fType = '3UTR'
                else:
                    fType = '5UTR'
                insertArray.append((geneID, 1, chrom, estart, gstop, sense, 'CDS'))
                insertArray.append((geneID, 1, chrom, gstop + 1, estop, sense, fType))
            else:
                if sense == 'F':
                    fType = '5UTR'
                else:
                    fType = '3UTR'
                insertArray.append((geneID, 1, chrom, gstart, estop, sense, 'CDS'))
                insertArray.append((geneID, 1, chrom, estart, gstart - 1, sense, fType))
    geneFile.close()
    
    mmGenome = Genome('mouse', dbFile=db)
    print 'Adding %d features' % len(insertArray)
    mmGenome.addFeatureEntryBatch(insertArray)

def loadGeneAnnotations(db, annotPath):
    geneAnnotations = []
    annotFile = open(annotPath, 'r')  
    locus = {}
    mmGenome = Genome('mouse', dbFile=db)
    
    for line in annotFile:
        try:
            cols = line.split('\t')
            locID = cols[6]
            geneDesc = cols[1] 
            
            if len(locID) > 0:
                geneAnnotations.append((('mouse', locID), string.replace(geneDesc.strip(), "'", "p")))
        except:
            pass
    
    print "Adding %d annotations" % len(geneAnnotations)
    mmGenome.addAnnotationBatch(geneAnnotations)

def loadGeneOntology(db, goPath, goDefPath, annotPath):
    mmGenome = Genome('mouse', dbFile=db)
    goDefFile = open(goDefPath, 'r')
    goFile = open(goPath, 'r')
    
    annotFile = open(annotPath, 'r')  
    annotEntries = annotFile.readlines()
    annotFile.close()
    goDefEntries = goDefFile.readlines()
    goDefs = {}
    locus = {}
    goArray = []
    
    for goDefEntry in goDefEntries:
        if goDefEntry[0] != '!':
            cols = goDefEntry.split('\t')
            goDefs[cols[0]] = (cols[1], cols[2].strip())
    
    goEntries = goFile.readlines()
    
    for annotEntry in annotEntries:
        try:
            cols = annotEntry.split('\t')
            locID = cols[6]
            geneName = cols[0]
            geneDesc = cols[1]
            
            mimID = ''
            if len(locID) > 0:
                locus[locID] = (geneName, geneDesc, '')
        except:
            pass
    
    for entry in goEntries:
        try:
            fields = entry.split('\t')
            if fields[0] != '10090':
                continue
            locID = fields[1].strip()
            (gene_name, gene_desc, mimID) = locus[locID]
            goArray.append((('mouse', locID), fields[2], '', gene_name, '', string.replace(goDefs[fields[2]][0], "'", "p"), goDefs[fields[2]][1], mimID))
        except:
            print "locus ID %s could not be added" % locID
            pass
    
    print "adding %d go entries" % len(goArray)
    mmGenome.addGoInfoBatch(goArray)

def createDBFile(db):
    mmGenome = Genome('mouse',  dbFile=db)
    mmGenome.createGeneDB(db)

def createDBindices(db):
    mmGenome = Genome('mouse', dbFile=db)
    mmGenome.createIndices()

def buildMouseDB(db=geneDB):
    genePath = '/proj/genome/download/refGene.txt' # ftp://hgdownload.cse.ucsc.edu/goldenPath/mm6/database/refGene.txt.gz
    annotPath='/proj/genome/download/refLink.txt' # ftp://hgdownload.cse.ucsc.edu/goldenPath/mm6/database/refLink.txt.gz
    goDefPath = '/proj/genome/download/GO.terms_and_ids' # ftp://ftp.geneontology.org/pub/go/doc/GO.terms_and_ids
    goPath = '/proj/genome/download/gene2go' # ftp://ftp.ncbi.nih.gov/gene/DATA/gene2go.gz
    
    # chromosomes are from ftp://hgdownload.cse.ucsc.edu/goldenPath/mm6/chromosomes
    chromos = {'1': '/proj/genome/download/chr1.fa', '2': '/proj/genome/download/chr2.fa', 
    '3': '/proj/genome/download/chr3.fa', '4': '/proj/genome/download/chr4.fa', 
    '5': '/proj/genome/download/chr5.fa', '6': '/proj/genome/download/chr6.fa', 
    '7': '/proj/genome/download/chr7.fa', '8': '/proj/genome/download/chr8.fa', 
    '9': '/proj/genome/download/chr9.fa', '10': '/proj/genome/download/chr10.fa', 
    '11': '/proj/genome/download/chr11.fa', '12': '/proj/genome/download/chr12.fa', 
    '13': '/proj/genome/download/chr13.fa', '14': '/proj/genome/download/chr14.fa', 
    '15': '/proj/genome/download/chr15.fa', '16': '/proj/genome/download/chr16.fa',
    '17': '/proj/genome/download/chr17.fa', '18': '/proj/genome/download/chr18.fa', 
    '19': '/proj/genome/download/chr19.fa', 'Un': '/proj/genome/download/chrUn_random.fa',  
    'X': '/proj/genome/download/chrX.fa', 'Y': '/proj/genome/download/chrY.fa',
    '1rand': '/proj/genome/download/chr1_random.fa',  
    '3rand': '/proj/genome/download/chr3_random.fa',  
    '5rand': '/proj/genome/download/chr5_random.fa', '6rand': '/proj/genome/download/chr6_random.fa', 
    '7rand': '/proj/genome/download/chr7_random.fa', '8rand': '/proj/genome/download/chr8_random.fa', 
    '9rand': '/proj/genome/download/chr9_random.fa', '10rand': '/proj/genome/download/chr10_random.fa',
    '12rand': '/proj/genome/download/chr12_random.fa', '13rand': '/proj/genome/download/chr13_random.fa', 
    '14rand': '/proj/genome/download/chr14_random.fa', '15rand': '/proj/genome/download/chr15_random.fa', 
    '16rand': '/proj/genome/download/chr16_random.fa', '17rand': '/proj/genome/download/chr17_random.fa', 
    '18rand': '/proj/genome/download/chr18_random.fa', '19rand': '/proj/genome/download/chr19_random.fa',  
    'Yrand': '/proj/genome/download/chrY_random.fa'}
    
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Adding gene entries"
    loadGeneEntries(db, genePath, annotPath)
    
    print "Adding gene features"
    loadGeneFeatures(db, genePath, annotPath)
    
    print "Adding gene annotations"
    loadGeneAnnotations(db, annotPath)
    
    print "Adding gene ontology"
    loadGeneOntology(db, goPath, goDefPath, annotPath)
    
    for chromID in chromos.keys():
        print "Loading chromosome %s" % chromID
        loadChromosome(db, chromID, chromos[chromID], '/proj/genome/M_musculus/chromo' + chromID + '.bin')
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db

def fixRandoms(refGene='/proj/genome/download/refGene.txt', refLink='/proj/genome/download/refLink.txt', db=geneDB):
    geneFile = open(refGene,'r')
    linkFile = open(refLink, 'r')
    
    randChrom = { 'chrUn_random': 'Un', 'chr1_random': '1rand', 'chr2_random': '2rand','chr3_random': '3rand',
    'chr4_random': '4rand','chr5_random': '5rand','chr6_random': '6rand','chr7_random': '7rand','chr8_random': '8rand',
    'chr9_random': '9rand','chr10_random': '10rand','chr11_random': '11rand','chr12_random': '12rand','chr13_random': '13rand',
    'chr14_random': '14rand','chr15_random': '15rand','chr16_random': '16rand','chr17_random': '17rand','chr18_random': '18rand',
    'chr19_random': '19rand'}
    senseArray = {'+': 'F', '-': 'R'}
    genesToFix = {}
    updateArray = []
    
    for line in geneFile:
        fields = line.split('\t')
        if fields[1] in randChrom:
            genesToFix[fields[0]] = (randChrom[fields[1]], senseArray[fields[2]], fields[3], fields[4])
    
    geneFile.close()
    
    for line in linkFile:
        fields = line.split('\t')
        if fields[2] in genesToFix:
            (chrom, sense, start, stop) = genesToFix[fields[2]]
            stmt = "UPDATE gene_entries set chromosome = '%s', start = '%s', stop='%s', orientation='%s' where name = '%s' " % (chrom, str(int(start) - 1),str(int(stop) - 1), sense, fields[6])
            updateArray.append(stmt)
    
    linkFile.close()
    
    print 'Fixing %d entries' % len(updateArray)
    mmGenome = Genome('mouse', dbFile=db)
    mmGenome.writeBatchDB(updateArray)

def fixRandomFeatures(refGene='/proj/genome/download/refGene.txt', refLink='/proj/genome/download/refLink.txt', db=geneDB):
    geneFile = open(refGene,'r')
    linkFile = open(refLink, 'r')
    
    randChrom = { 'chrUn_random': 'Un', 'chr1_random': '1rand', 'chr2_random': '2rand','chr3_random': '3rand',
    'chr4_random': '4rand','chr5_random': '5rand','chr6_random': '6rand','chr7_random': '7rand','chr8_random': '8rand',
    'chr9_random': '9rand','chr10_random': '10rand','chr11_random': '11rand','chr12_random': '12rand','chr13_random': '13rand',
    'chr14_random': '14rand','chr15_random': '15rand','chr16_random': '16rand','chr17_random': '17rand','chr18_random': '18rand',
    'chr19_random': '19rand'}
    senseArray = {'+': 'F', '-': 'R'}
    genesToFix = {}
    deleteArray = []
    insertArray = []
    startDict = {}
    stopDict = {}
    
    for line in geneFile:
        fields = line.split('\t')
        if fields[1] in randChrom:
            genesToFix[fields[0]] = line
            startDict[fields[0]] = int(fields[5])
            stopDict[fields[0]] = int(fields[6])
    
    geneFile.close()
    
    for line in linkFile:
        fields = line.split('\t')
        if fields[2] in genesToFix:
            stmt = "Delete from sequence_features where name = '%s' " % fields[6]
            deleteArray.append(stmt)
            geneLine = genesToFix[fields[2]]
            geneFields = geneLine.split('\t')
            exonNum = int(geneFields[7])
            exonStarts = geneFields[8].split(',')
            exonStops = geneFields[9].split(',')
            chrom = randChrom[geneFields[1]]
            sense = senseArray[geneFields[2]]
            gstop = stopDict[fields[2]] - 1
            gstart = startDict[fields[2]] - 1
            geneID = ('mouse', fields[6])
            for index in range(exonNum):
                estart = int(exonStarts[index]) - 1
                estop = int(exonStops[index]) - 1
                if estart >= gstart and estop <= gstop:
                    insertArray.append((geneID, 1, chrom, estart, estop, sense, 'CDS'))
                elif estop <= gstart:
                    if sense == 'F':
                        fType = '5UTR'
                    else:
                        fType = '3UTR'
                    insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
                elif estart >= gstop:
                    if sense == 'F':
                        fType = '3UTR'
                    else:
                        fType = '5UTR'
                    insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
                elif estart <= gstop:
                    if sense == 'F':
                        fType = '3UTR'
                    else:
                        fType = '5UTR'
                    insertArray.append((geneID, 1, chrom, estart, gstop, sense, 'CDS'))
                    insertArray.append((geneID, 1, chrom, gstop + 1, estop, sense, fType))
                else:
                    if sense == 'F':
                        fType = '5UTR'
                    else:
                        fType = '3UTR'
                    insertArray.append((geneID, 1, chrom, gstart, estop, sense, 'CDS'))
                    insertArray.append((geneID, 1, chrom, estart, gstart - 1, sense, fType))

    
    linkFile.close()
    
    print 'Fixing %d genes' % len(deleteArray)
    mmGenome = Genome('mouse', dbFile=db)
    mmGenome.writeBatchDB(deleteArray)
    print 'Adding %d features' % len(insertArray)
    mmGenome.addFeatureEntryBatch(insertArray)

    