###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Petromyzon marinus
import os, string
from cistematic.genomes import Genome
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB = cisRoot + '/P_marinus/pmarinus.genedb' 

def loadChromosomes(db, chromPath, chromOutPath):
    sequenceList = []
    addSequence = 0
    chromosomeList = []
    pmGenome = Genome('pmarinus', dbFile=db)
    
    files = os.listdir(chromPath)
    for filename in files:
        inFile = open(chromPath + '/' + filename, 'r')
        # skip the header
        index = 0
        header = inFile.readline()
        while header != "":
            seqArray = []
            seqLen = 0
            chromLine = header.strip().split()
            chromID = chromLine[1]
            currentLine = inFile.readline()
            while currentLine != "" and currentLine[0] != ">":
                lineSeq = currentLine.strip()
                seqLen += len(lineSeq)
                seqArray.append(lineSeq)
                currentLine = inFile.readline()
            seq = string.join(seqArray,'')
            if seqLen < 250000:
                print 'Added contig %s to database' % chromID
                sequenceList.append((('pmarinus', chromID), seq, 'chromosome', str(seqLen)))
                chromosomeList.append((chromID, chromID, 'db'))
                addSequence += 1
            else:
                outFileName = chromOutPath + chromID + '.bin'
                outFile = open(cisRoot + outFileName, 'w')
                outFile.write(seq)
                outFile.close()
                print 'Added contig file %s to database' % outFileName
                chromosomeList.append((chromID, outFileName, 'file'))
            header = currentLine
            if addSequence > 1000: 
                pmGenome.addSequenceBatch(sequenceList)
                sequenceList = []
                addSequence = 0
        inFile.close()
    if addSequence > 0:
        pmGenome.addSequenceBatch(sequenceList)
    pmGenome.addChromosomeEntryBatch(chromosomeList)

def createDBFile(db):
    pmGenome = Genome('pmarinus',  dbFile=db)
    pmGenome.createGeneDB(db)

def createDBindices(db):
    pmGenome = Genome('pmarinus', dbFile=db)
    pmGenome.createIndices()

def buildPmarinusDB(db=geneDB):
    chromoPath = cisRoot + '/download/pm'
    chromoOutPath = '/P_marinus/'
        
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Loading sequences" 
    loadChromosomes(db, chromoPath, chromoOutPath)
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db
