###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Ratus Norvegicus
import string
from cistematic.genomes import Genome
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB  = cisRoot + '/R_norvegicus/rat.genedb' 

def loadChromosome(db, chromID, chromPath, chromOut):
    seq = ''
    seqArray = []
    seqLen = 0
    rnGenome = Genome('rat', dbFile=db)
    
    inFile = open(chromPath, 'r')
    # skip the header
    index = 0
    line = inFile.readline()
    for line in inFile:
        seqArray.append(line.strip())
    
    seq = string.join(seqArray,'')
    seqLen = len(seq)
    if seqLen < 1:
        print "Problems reading sequence from file"
    print "writing to file %s" % chromOut
    outFile = open(cisRoot + chromOut,'w')
    outFile.write(seq)
    outFile.close()
    
    #print "calling rnGenome()"
    rnGenome.addChromosomeEntry(chromID, chromOut, 'file')

def loadGeneEntries(db, gFile):
    """ FIXME - NEED TO DEAL WITH ALTERNATIVE SPLICING ENTRIES
    """
    geneEntries = []
    rnGenome = Genome('rat', dbFile=db)
    geneFile = open(gFile, 'r')
    
    for line in geneFile:
        cols = line.split('\t')
        if cols[11] != 'GENE':
            continue
        if cols[12] == 'Celera':
            continue
        
        name = cols[10].split(':')
        gid = name[1]
        start = int(cols[2]) - 1
        stop = int(cols[3]) - 1
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('rat', gid)
        gidVersion = 1
        geneEntries.append((geneID, chrom, start, stop, sense, 'gene', gidVersion))
    
    print "Adding %d gene entries" % len(geneEntries)
    rnGenome.addGeneEntryBatch(geneEntries)

def loadGeneFeatures(db, gFile):
    """ Load gene features such as CDS, UTR, and PSEUDO from the gene file.
    """
    featureEntries = []
    rnGenome = Genome('rat', dbFile=db)
    featureFile = open(gFile, 'r')
    
    for line in featureFile:
        cols = line.split('\t')
        if cols[11] not in ['CDS', 'UTR', 'PSEUDO']:
            continue
        if cols[12] == 'Celera':
            continue
        
        fType = cols[11]
        name = cols[10].split(':')
        gid = name[1]
        start = int(cols[2]) - 1
        stop = int(cols[3]) - 1
        sense = cols[4]
        chrom = cols[1].strip()
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('rat', gid)
        gidVersion = 1
        featureEntries.append((geneID, gidVersion, chrom, start, stop, sense, fType))
    
    print "Adding %d feature entries" % len(featureEntries)
    rnGenome.addFeatureEntryBatch(featureEntries)

def loadGeneAnnotations(db, annotPath):
    geneAnnotations = []
    annotFile = open(annotPath, 'r')  
    locus = {}
    rnGenome = Genome('rat', dbFile=db)
    
    for line in annotFile:
        try:
            cols = line.split('\t')
            locID = cols[0]
            geneName = cols[1]
            geneDesc = cols[6]
            
            if len(locID) > 0:
                geneAnnotations.append((('rat', locID), string.replace(geneDesc.strip(), "'", "p")))
        except:
            pass
    
    print "Adding %d annotations" % len(geneAnnotations)
    rnGenome.addAnnotationBatch(geneAnnotations)

def loadGeneOntology(db, goPath, goDefPath, annotPath):
    rnGenome = Genome('rat', dbFile=db)
    goDefFile = open(goDefPath, 'r')
    goFile = open(goPath, 'r')
    
    annotFile = open(annotPath, 'r')  
    annotEntries = annotFile.readlines()
    annotFile.close()
    goDefEntries = goDefFile.readlines()
    goDefs = {}
    locus = {}
    goArray = []
    
    for goDefEntry in goDefEntries:
        if goDefEntry[0] != '!':
            cols = goDefEntry.split('\t')
            goDefs[cols[0]] = (cols[1], cols[2].strip())
    
    goEntries = goFile.readlines()
    
    for annotEntry in annotEntries:
        try:
            cols = annotEntry.split('\t')
            locID = cols[0]
            geneName = cols[1]
            geneDesc = cols[6]
            
            mimID = ''
            if len(locID) > 0:
                locus[locID] = (geneName, geneDesc, mimID)
        except:
            pass
    
    for entry in goEntries:
        try:
            fields = entry.split('\t')
            locID = fields[0].strip()
            (gene_name, gene_desc, mimID) = locus[locID]
            goArray.append((('rat', locID), fields[1], '', gene_name, '', string.replace(goDefs[fields[1]][0], "'", "p"), goDefs[fields[1]][1], mimID))
        except:
            #print "locus ID %s could not be added" % locID
            pass
    
    print "adding %d go entries" % len(goArray)
    rnGenome.addGoInfoBatch(goArray)

def createDBFile(db):
    rnGenome = Genome('rat',  dbFile=db)
    rnGenome.createGeneDB(db)

def createDBindices(db):
    rnGenome = Genome('rat', dbFile=db)
    rnGenome.createIndices()

def buildRatDB(db=geneDB):
    genePath = cisRoot + '/download/seq_gene.md'
    annotPath=cisRoot + '/download/LL.out_rn'
    goDefPath = cisRoot + '/download/GO.terms_and_ids'
    goPath = cisRoot + '/download/loc2go'
    
    chromos = {'1': cisRoot + '/download/chr1.fa', '2': cisRoot + '/download/chr2.fa', 
    '3': cisRoot + '/download/chr3.fa', '4': cisRoot + '/download/chr4.fa', 
    '5': cisRoot + '/download/chr5.fa', '6': cisRoot + '/download/chr6.fa', 
    '7': cisRoot + '/download/chr7.fa', '8': cisRoot + '/download/chr8.fa', 
    '9': cisRoot + '/download/chr9.fa', '10': cisRoot + '/download/chr10.fa', 
    '11': cisRoot + '/download/chr11.fa', '12': cisRoot + '/download/chr12.fa', 
    '13': cisRoot + '/download/chr13.fa', '14': cisRoot + '/download/chr14.fa', 
    '15': cisRoot + '/download/chr15.fa', '16': cisRoot + '/download/chr16.fa',
    '17': cisRoot + '/download/chr17.fa', '18': cisRoot + '/download/chr18.fa', 
    '19': cisRoot + '/download/chr19.fa', 'Un': cisRoot + '/download/chrUn.fa',  
    'X': cisRoot + '/download/chrX.fa', '20': cisRoot + '/download/chr20.fa',
    '1rand': cisRoot + '/download/chr1_random.fa', '2rand': cisRoot + '/download/chr2_random.fa', 
    '3rand': cisRoot + '/download/chr3_random.fa', '4rand': cisRoot + '/download/chr4_random.fa', 
    '5rand': cisRoot + '/download/chr5_random.fa', '6rand': cisRoot + '/download/chr6_random.fa', 
    '7rand': cisRoot + '/download/chr7_random.fa', '8rand': cisRoot + '/download/chr8_random.fa', 
    '9rand': cisRoot + '/download/chr9_random.fa', '10rand': cisRoot + '/download/chr10_random.fa', 
    '11rand': cisRoot + '/download/chr11_random.fa',  'Unrand': cisRoot + '/download/chrUn_random.fa', 
    '12rand': cisRoot + '/download/chr12_random.fa', '13rand': cisRoot + '/download/chr13_random.fa', 
    '14rand': cisRoot + '/download/chr14_random.fa', '15rand': cisRoot + '/download/chr15_random.fa', 
    '16rand': cisRoot + '/download/chr16_random.fa', '17rand': cisRoot + '/download/chr17_random.fa', 
    '18rand': cisRoot + '/download/chr18_random.fa', '19rand': cisRoot + '/download/chr19_random.fa',  
    'Xrand': cisRoot + '/download/chrX_random.fa' , '20rand': cisRoot + '/download/chr20_random.fa'}
    
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Adding gene entries"
    loadGeneEntries(db, genePath)
    
    print "Fixing Random Genes"
    fixRandoms()
    
    print "Adding gene features"
    loadGeneFeatures(db, genePath)
    
    print "Fixing random features"
    fixRandomFeatures()
    
    print "Adding gene annotations"
    loadGeneAnnotations(db, annotPath)
    
    print "Adding gene ontology"
    loadGeneOntology(db, goPath, goDefPath, annotPath)
    
    for chromID in chromos.keys():
        print "Loading chromosome %s" % chromID
        loadChromosome(db, chromID, chromos[chromID], '/R_norvegicus/chromo' + chromID + '.bin')
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db

def fixRandoms(refGene=cisRoot + '/download/refGene.txt', refLink=cisRoot + '/download/refLink.txt', db=geneDB):
    geneFile = open(refGene,'r')
    linkFile = open(refLink, 'r')
    
    randChrom = { 'chrUn': 'Un', 'chrUn_random': 'Unrand', 'chr1_random': '1rand', 'chr2_random': '2rand','chr3_random': '3rand',
    'chr4_random': '4rand','chr5_random': '5rand','chr6_random': '6rand','chr7_random': '7rand','chr8_random': '8rand',
    'chr9_random': '9rand','chr10_random': '10rand','chr11_random': '11rand','chr12_random': '12rand','chr13_random': '13rand',
    'chr14_random': '14rand','chr15_random': '15rand','chr16_random': '16rand','chr17_random': '17rand','chr18_random': '18rand',
    'chr19_random': '19rand', 'chr20_random': '20rand', 'chrX_random': 'Xrand'}
    senseArray = {'+': 'F', '-': 'R'}
    genesToFix = {}
    updateArray = []
    
    for line in geneFile:
        fields = line.split('\t')
        if fields[1] in randChrom:
            genesToFix[fields[0]] = (randChrom[fields[1]], senseArray[fields[2]], fields[3], fields[4])
    
    geneFile.close()
    
    for line in linkFile:
        fields = line.split('\t')
        if fields[2] in genesToFix:
            (chrom, sense, start, stop) = genesToFix[fields[2]]
            stmt = "UPDATE gene_entries set chromosome = '%s', start = '%s', stop='%s', orientation='%s' where name = '%s' " % (chrom, str(int(start) - 1), str(int(stop) - 1), sense, fields[6])
            updateArray.append(stmt)
    
    linkFile.close()
    
    print 'Fixing %d entries' % len(updateArray)
    rnGenome = Genome('rat', dbFile=db)
    rnGenome.writeBatchDB(updateArray)

def fixRandomFeatures(refGene=cisRoot + '/download/refGene.txt', refLink=cisRoot + '/download/refLink.txt', db=geneDB):
    geneFile = open(refGene,'r')
    linkFile = open(refLink, 'r')
    
    randChrom = { 'chrUn': 'Un', 'chrUn_random': 'Unrand', 'chr1_random': '1rand', 'chr2_random': '2rand','chr3_random': '3rand',
    'chr4_random': '4rand','chr5_random': '5rand','chr6_random': '6rand','chr7_random': '7rand','chr8_random': '8rand',
    'chr9_random': '9rand','chr10_random': '10rand','chr11_random': '11rand','chr12_random': '12rand','chr13_random': '13rand',
    'chr14_random': '14rand','chr15_random': '15rand','chr16_random': '16rand','chr17_random': '17rand','chr18_random': '18rand',
    'chr19_random': '19rand', 'chr20_random': '20rand', 'chrX_random': 'Xrand'}
    senseArray = {'+': 'F', '-': 'R'}
    genesToFix = {}
    deleteArray = []
    insertArray = []
    startDict = {}
    stopDict = {}
    
    for line in geneFile:
        fields = line.split('\t')
        if fields[1] in randChrom:
            genesToFix[fields[0]] = line
            startDict[fields[0]] = int(fields[5])
            stopDict[fields[0]] = int(fields[6])
    
    geneFile.close()
    
    for line in linkFile:
        fields = line.split('\t')
        if fields[2] in genesToFix:
            stmt = "Delete from sequence_features where name = '%s' " % fields[6]
            deleteArray.append(stmt)
            geneLine = genesToFix[fields[2]]
            geneFields = geneLine.split('\t')
            exonNum = int(geneFields[7])
            exonStarts = geneFields[8].split(',')
            exonStops = geneFields[9].split(',')
            chrom = randChrom[geneFields[1]]
            sense = senseArray[geneFields[2]]
            gstop = stopDict[fields[2]] - 1
            gstart = startDict[fields[2]] - 1
            geneID = ('rat', fields[6])
            for index in range(exonNum):
                estart = int(exonStarts[index]) - 1
                estop = int(exonStops[index]) - 1
                if estart >= gstart and estop <= gstop:
                    insertArray.append((geneID, 1, chrom, estart, estop, sense, 'CDS'))
                elif estop <= gstart:
                    if sense == 'F':
                        fType = '5UTR'
                    else:
                        fType = '3UTR'
                    insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
                elif estart >= gstop:
                    if sense == 'F':
                        fType = '3UTR'
                    else:
                        fType = '5UTR'
                    insertArray.append((geneID, 1, chrom, estart, estop, sense, fType))
                elif estart <= gstop:
                    if sense == 'F':
                        fType = '3UTR'
                    else:
                        fType = '5UTR'
                    insertArray.append((geneID, 1, chrom, estart, gstop, sense, 'CDS'))
                    insertArray.append((geneID, 1, chrom, gstop + 1, estop, sense, fType))
                else:
                    if sense == 'F':
                        fType = '5UTR'
                    else:
                        fType = '3UTR'
                    insertArray.append((geneID, 1, chrom, gstart, estop, sense, 'CDS'))
                    insertArray.append((geneID, 1, chrom, estart, gstart - 1, sense, fType))

    
    linkFile.close()
    
    print 'Fixing %d genes' % len(deleteArray)
    rnGenome = Genome('rat', dbFile=db)
    rnGenome.writeBatchDB(deleteArray)
    print 'Adding %d features' % len(insertArray)
    rnGenome.addFeatureEntryBatch(insertArray)

