###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Orizia Sativa
import string
from cistematic.genomes import Genome
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB = cisRoot + '/O_sativa/osativa.genedb' 

chromSize = {'1':30432563, '2': 19705359,  '3': 23470805, '4': 18585042, '5': 26992738}

background = [0.3180185, 0.1819815, 0.1819815, 0.3180185]
genomeSize = 119186497

def loadChromosome(db, chromID, chromPath, chromOut):
    seq = ''
    seqArray = []
    seqLen = 0
    atGenome = Genome('osativa', dbFile=db)
    
    inFile = open(chromPath, 'r')
    # skip the header
    index = 0
    line = inFile.readline()
    for line in inFile:
        seqArray.append(line.strip())
    
    seq = string.join(seqArray,'')
    seqLen = len(seq)
    if seqLen < 1:
        print "Problems reading sequence from file"
    print "writing to file %s" % chromOut
    outFile = open(cisRoot + chromOut,'w')
    outFile.write(seq)
    outFile.close()
    seq = ''
    
    osGenome.addChromosomeEntry(chromID, chromOut, 'file')
    osGenome.addChromosomeEntry('chromo' + chromID, chromOut, 'file')

def loadGeneEntries(db, gFile):
    geneEntries = []
    osGenome = Genome('osativa', dbFile=db)
    geneFile = open(gFile, 'r')
    
    geneDict = {}
    ORFDict = {}
    
    for line in geneFile:
        if line[0] == 'c' or line[0]=='\n':
            continue
        fields = line.split('\t')
        #print str(fields)
        locusField = fields[1].split('.')
        try:
            (locus, rev) = locusField
        except:
            print 'non standard geneID: %s' % (locusField)
            locus = locusField[0]
            rev = '0'
        if int(rev) != 1:
            locus = locus + '.' + rev
        if fields[3].strip() == 'GENE':
            geneDict[locus] = line
        if fields[3].strip() == 'ORF':
            ORFDict[locus] = line
    
    ORFKeys = ORFDict.keys()
    ORFKeys.sort()
    
    geneKeys = geneDict.keys()
    geneKeys.sort()
    
    for orf in ORFKeys:
        line = ORFDict[orf]
        field = line.split('\t')
        try:
            gidrev = field[1].split('.')
            gid = gidrev[0]
            version = gidrev[1]
        except:
            gid = field[1]
            version = '1'
        length = int(field[6])
        
        sense = field[7][0]
        chrom = field[0].strip()
        if sense == 'r':
            start = field[5].strip()
            stop  = field[4].strip()
        else:
            start = field[4].strip()
            stop  = field[5].strip()
        
        if int(start) > int(stop):
            temp = start
            start = stop
            stop = temp
        
        if sense == 'f':
            start = str(int(start) - 1)
            stop = str(int(stop) - 1)
        
        geneID = ('osativa', gid)
        geneEntries.append((geneID, chrom, start, stop, sense.upper(), 'ORF', version))
    
    print "inserted %d gene entries" % len(geneEntries)
    osGenome.addGeneEntryBatch(geneEntries)

def loadFeatureEntries(db, gFile):
    featureEntries = []
    osGenome = Genome('osativa', dbFile=db)
    geneFile = open(gFile, 'r')
    
    geneFile = open(gFile, 'r')
    for line in geneFile:
        if line[0] == 'c' or line[0]=='\n':
            continue
        fields = line.split('\t')
        #print str(fields)
        locusField = fields[1].split('.')
        try:
            (locus, rev) = locusField
        except:
            print 'non standard geneID: %s' % (locusField)
            locus = locusField[0]
            rev = '0'
        if int(rev) != 1:
            locus = locus + '.' + rev
        if fields[3].strip() != 'coding_region':
            continue
        
        try:
            gidrev = fields[1].split('.')
            gid = gidrev[0]
            version = gidrev[1]
        except:
            gid = fields[1]
            version = '1'
        length = int(fields[6])
        
        sense = fields[7][0]
        chrom = fields[0].strip()
        if sense == 'r':
            start = fields[5].strip()
            stop  = fields[4].strip()
        else:
            start = fields[4].strip()
            stop  = fields[5].strip()
        
        if int(start) > int(stop):
            temp = start
            start = stop
            stop = temp
        
        if sense == 'f':
            start = str(int(start) - 1)
            stop = str(int(stop) - 1)
        
        geneID = ('osativa', gid)
        featureEntries.append((geneID, version, chrom, start, stop, sense.upper(), 'CDS'))
    
    print "inserted %d feature entries" % len(featureEntries)
    osGenome.addFeatureEntryBatch(featureEntries)

def createDBFile(db):
    osGenome = Genome('osativa', dbFile=db)
    osGenome.createGeneDB(db)

def createDBindices(db):
    osGenome = Genome('osativa', dbFile=db)
    osGenome.createIndices()

def buildRiceDB(db=geneDB):
    genePath = cisRoot + '/download/9311_FgeneSH20040801.NoTE.Ann'
    
    chromos = {'1': cisRoot + '/download/Chr01.fasta', '2': cisRoot + '/download/Chr02.fasta', 
    '3': cisRoot + '/download/Chr03.fasta', '4': cisRoot + '/download/Chr04.fasta', 
    '5': cisRoot + '/download/Chr05.fasta', '6': cisRoot + '/download/Chr06.fasta', 
    '7': cisRoot + '/download/Chr07.fasta', '8': cisRoot + '/download/Chr08.fasta', 
    '9': cisRoot + '/download/Chr09.fasta', '10': cisRoot + '/download/Chr10.fasta', 
    '11': cisRoot + '/download/Chr11.fasta', '12': cisRoot + '/download/Chr12.fasta', 
    'Un': cisRoot + '/download/ChrUn.fasta' 
      }
    
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Adding gene entries"
    loadGeneEntries(db, genePath)
    
    print "Adding feature entries"
    loadFeatureEntries(db, genePath)
    
    for chromID in ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', 'Un']:
        print "Loading chromosome %s" % chromID
        loadChromosome(db, chromID, chromos[chromID], '/O_sativa/chr' + chromID + '.bin')
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db
