###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Saccharomyces Mikatae
import string, cistematic.core.homology
from cistematic.genomes import Genome
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB = cisRoot + '/S_mikatae/smikatae.genedb' 

def loadChromosome(db, chromPath):
    seqArray = []
    seqLen = 0
    seqList = []
    chromList = []
    smGenome = Genome('smikatae', dbFile=db)
    
    print "loading contigs"
    inFile = open(chromPath, 'r')
    # skip the header
    index = 0
    header = inFile.readline()
    while header != "":
        seqArray = []
        seqLen = 0
        headline = header.split('[')
        chromID = headline[0][1:]
        currentLine = inFile.readline()
        
        while currentLine != "" and currentLine[0] != ">":
            lineSeq = currentLine.strip()
            seqLen += len(lineSeq)
            seqArray.append(lineSeq)
            currentLine = inFile.readline()
        
        seq = string.join(seqArray,'')
        #print 'Added contig %s (%d bp) to database' % (chromID, seqLen)
        seqList.append((('smikatae', chromID), seq, 'chromosome', str(seqLen)))
        chromList.append((chromID, chromID, 'db'))

        header = currentLine
    
    inFile.close()
    smGenome.addSequenceBatch(seqList)
    smGenome.addChromosomeEntryBatch(chromList)

def loadGeneEntries(db, gDict):
    geneEntries = []
    geneFeatures = []
    smGenome = Genome('smikatae', dbFile=db)
    
    for orfName in gDict:
        (chrom, start, stop, sense) = gDict[orfName]
        geneID = ('smikatae', orfName)
        gidVersion = 1
        
        geneEntries.append((geneID, chrom, start, stop, sense, 'chromosomal_feature', gidVersion))
        geneFeatures.append((geneID, gidVersion, chrom, start, stop, sense, 'CDS'))
    
    print "loading %d gene entries" % len(geneEntries)
    smGenome.addGeneEntryBatch(geneEntries)
    print "loading %d gene features" % len(geneFeatures)
    smGenome.addFeatureEntryBatch(geneFeatures)

def loadGeneAnnotations(db, aDict):
    geneAnnotations = []
    smGenome = Genome('smikatae', dbFile=db)
    
    for orfName in aDict:
        geneAnnotations.append((('smikatae', orfName), string.replace(aDict[orfName], "'", "p")))
    
    print "Adding %d annotations" % len(geneAnnotations)
    smGenome.addAnnotationBatch(geneAnnotations)

def createDBFile(db):
    smGenome = Genome('smikatae', version='SGD1',  dbFile=db)
    smGenome.createGeneDB(db)

def createDBindices(db):
    smGenome = Genome('smikatae', version='SGD1', dbFile=db)
    smGenome.createIndices()

def processOrfFasta(db, filename):
    smGenome = Genome('smikatae', version='SGD1', dbFile=db)
    orfRes = {}
    annotRes = {}
    dictKey = ''
    counter = 0
    theFile = open(filename, 'r')
    
    for line in theFile:
        if line[0] == '>':
            tempLine = line[1:-1].split(',')
            head = tempLine[0].split()
            tail = tempLine[1].split()
            contig = tail[1]
            (beg, end) = tail[2].split('-')
            
            start = int(beg) - 1
            stop  = int(end) - 1
            
            dictKey = head[0][5:]
            
            if len(tail) > 3:
                firstCodon = smGenome.sequence(contig, start, 3)
                if firstCodon == 'ATG':
                    counter += 1
                    sense = 'F'
                else:
                    sense = 'R'
            else:
                lastCodon = smGenome.sequence(contig, stop - 2, 3)
                if lastCodon == 'CAT':
                    sense = 'R'
                else:
                    sense = 'F'
            
            try:
                if len(head[1]) > 0:
                    annotRes[dictKey] = head[1]
            except:
                pass
            
            orfRes[dictKey] = (contig, start, stop, sense)
        else:
            continue
    
    theFile.close()
    
    return (orfRes, annotRes)

def buildSmikataeDB(db=geneDB):
    contigPath = cisRoot + '/download/contig.fasta'
    orfPath = cisRoot + '/download/orf_genomic.fasta'
    
    print "Creating database %s" % db
    createDBFile(db)
    
    loadChromosome(db, contigPath)
    
    print "Building ORF dictionary"
    (orfDict, annotationDict) = processOrfFasta(db, orfPath)
    
    loadGeneEntries(db, orfDict)
    loadGeneAnnotations(db, annotationDict)
        
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db

def addSmikataeOrthologs(infilename= cisRoot + '/download/orf_genomic.fasta'):
    infile = open(infilename, 'r')
    mapList = []
    for line in infile:
        if line[0] != '>':
            continue
        fields = line.split(',')
        headfields = fields[0].split()
        if len(headfields) == 1:
            continue
        spid = headfields[0][6:]
        scid = headfields[1]
        if 'Multiple' in scid:
            continue
        if scid[-1] != 'C' and scid[-1] != 'W':
            continue
        mapList.append((scid, spid))
    infile.close()
    cistematic.core.homology.addHomologs('scerevisiae', 'smikatae', mapList)

