###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-5 by:                                               #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Tetraodon nigroviridis
import string
from cistematic.genomes import Genome
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB = cisRoot + '/T_nigroviridis/tnigroviridis.genedb' 

def loadChromosome(db, chromID, chromPath, chromOut):
    seq = ''
    seqArray = []
    seqLen = 0
    tnGenome = Genome('tnigroviridis', dbFile=db)
    
    inFile = open(chromPath, 'r')
    # skip the header
    index = 0
    line = inFile.readline()
    for line in inFile:
        seqArray.append(line.strip())
    
    seq = string.join(seqArray,'')
    seqLen = len(seq)
    if seqLen < 1:
        print "Problems reading sequence from file"
    print "writing to file %s" % chromOut
    outFile = open(cisRoot + chromOut,'w')
    outFile.write(seq)
    outFile.close()
    
    #print "calling tnGenome()"
    tnGenome.addChromosomeEntry(chromID, chromOut, 'file')

def loadGeneEntries(db, gFile):
    """ FIXME - NEED TO DEAL WITH ALTERNATIVE SPLICING ENTRIES
    """
    geneEntries = []
    tnGenome = Genome('tnigroviridis', dbFile=db)
    geneFile = open(gFile, 'r')
    
    for line in geneFile:
        cols = line.split('\t')
        
        gid = cols[0]
        start = int(cols[5])
        stop = int(cols[6])
        sense = cols[2]
        chrom = cols[1][3:]
        if sense == '+':
            sense = 'F'
        else:
            sense = 'R'
        
        geneID = ('tnigroviridis', gid)
        gidVersion = 1
        geneEntries.append((geneID, chrom, start, stop, sense, 'gene', gidVersion))
    
    print "Adding %d gene entries" % len(geneEntries)
    tnGenome.addGeneEntryBatch(geneEntries)

def loadGeneFeatures(db, gfile):
    geneFile = open(gfile,'r')
    
    senseDict = {'+': 'F', '-': 'R', '.':'F'}
    genesToFix = {}
    deleteArray = []
    insertArray = []
    startDict = {}
    stopDict = {}
    
    for geneLine in geneFile:
        geneFields = geneLine.split('\t')
        exonNum = int(geneFields[7])
        exonStarts = geneFields[8].split(',')
        exonStops = geneFields[9].split(',')
        chrom = geneFields[1][3:]
        sense = senseDict[geneFields[2]]
        gstart = int(geneFields[5]) - 1
        gstop = int(geneFields[6]) - 1
        name = geneFields[0]
        geneID = ('tnigroviridis', name[0])
        gidVersion = '1'
        
        for index in range(exonNum):
            estart = int(exonStarts[index]) - 1
            estop = int(exonStops[index]) - 1
            if estart >= gstart and estop <= gstop:
                insertArray.append((geneID, gidVersion, chrom, estart, estop, sense, 'CDS'))
            elif estop <= gstart:
                if sense == 'F':
                    fType = '5UTR'
                else:
                    fType = '3UTR'
                insertArray.append((geneID, gidVersion, chrom, estart, estop, sense, fType))
            elif estart >= gstop:
                if sense == 'F':
                    fType = '3UTR'
                else:
                    fType = '5UTR'
                insertArray.append((geneID, gidVersion, chrom, estart, estop, sense, fType))
            elif estart <= gstop and estart > gstart:
                if sense == 'F':
                    fType = '3UTR'
                else:
                    fType = '5UTR'
                insertArray.append((geneID, gidVersion, chrom, estart, gstop, sense, 'CDS'))
                insertArray.append((geneID, gidVersion, chrom, gstop + 1, estop, sense, fType))
            elif estart < gstart and estop <= gstop:
                if sense == 'F':
                    fType = '5UTR'
                else:
                    fType = '3UTR'
                insertArray.append((geneID, gidVersion, chrom, estart, gstart - 1, sense, fType))
                insertArray.append((geneID, gidVersion, chrom, gstart, estop, sense, 'CDS'))
            else:
                if sense == 'F':
                    fType1 = '5UTR'
                    fType2 = '3UTR'
                else:
                    fType1 = '3UTR'
                    fType2 = '5UTR'
                insertArray.append((geneID, gidVersion, chrom, estart, gstart - 1, sense, fType1))
                insertArray.append((geneID, gidVersion, chrom, gstart, gstop, sense, 'CDS'))
                insertArray.append((geneID, gidVersion, chrom, gstop + 1, estop - 1, sense, fType2))
                
    geneFile.close()
    
    tnGenome = Genome('tnigroviridis', dbFile=db)
    print 'Adding %d features' % len(insertArray)
    tnGenome.addFeatureEntryBatch(insertArray)

def createDBFile(db):
    tnGenome = Genome('tnigroviridis',  dbFile=db)
    tnGenome.createGeneDB(db)

def createDBindices(db):
    tnGenome = Genome('tnigroviridis', dbFile=db)
    tnGenome.createIndices()

def buildtnigroviridisDB(db=geneDB):
    genePath = cisRoot + '/download/geneid.txt'
    
    chromos = {'1': cisRoot + '/download/chr1.fa', '2': cisRoot + '/download/chr2.fa', 
    '3': cisRoot + '/download/chr3.fa', '4': cisRoot + '/download/chr4.fa', 
    '5': cisRoot + '/download/chr5.fa', '6': cisRoot + '/download/chr6.fa', 
    '7': cisRoot + '/download/chr7.fa', '8': cisRoot + '/download/chr8.fa', 
    '9': cisRoot + '/download/chr9.fa', '10': cisRoot + '/download/chr10.fa', 
    '11': cisRoot + '/download/chr11.fa', '12': cisRoot + '/download/chr12.fa', 
    '13': cisRoot + '/download/chr13.fa', '14': cisRoot + '/download/chr14.fa', 
    '15': cisRoot + '/download/chr15.fa', '16': cisRoot + '/download/chr16.fa',
    '17': cisRoot + '/download/chr17.fa', '18': cisRoot + '/download/chr18.fa', 
    '19': cisRoot + '/download/chr19.fa', '20': cisRoot + '/download/chr20.fa',  
    '21': cisRoot + '/download/chr21.fa', 
    '1_random': cisRoot + '/download/chr1_random.fa', '2_random': cisRoot + '/download/chr2_random.fa', 
    '3_random': cisRoot + '/download/chr3_random.fa', '15_random': cisRoot + '/download/chr15_random.fa', 
    '21_random': cisRoot + '/download/chr21_random.fa', 'Un_random': cisRoot + '/download/chrUn_random.fa'
    }
    
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Adding gene entries"
    loadGeneEntries(db, genePath)
    
    print "Adding gene features"
    loadGeneFeatures(db, genePath)
    
    for chromID in chromos.keys():
        print "Loading chromosome %s" % chromID
        loadChromosome(db, chromID, chromos[chromID], '/T_nigroviridis/chromo' + chromID + '.bin')
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db
